@@ -48,6 +48,7 @@ from wuttjamaican.app import (AppHandler as WuttaAppHandler,
 from rattail.util import (load_entry_points,
                           progress_loop, prettify,
                           pretty_quantity,
+                          render_duration,
                           NOTSET)
 from rattail.files import temp_path, resource_path
 from rattail.mail import send_email
@@ -77,7 +78,7 @@ class AppHandler(WuttaAppHandler):
     """
     default_app_title = "Rattail"
     default_model_spec = 'rattail.db.model'
-    default_auth_handler_spec = 'rattail.auth:AuthHandler'
+    default_auth_handler_spec = 'rattail.auth:RattailAuthHandler'
     default_people_handler_spec = 'rattail.people:PeopleHandler'
     default_autocompleters = {
         'brands': 'rattail.autocomplete.brands:BrandAutocompleter',
@@ -129,22 +130,28 @@ class AppHandler(WuttaAppHandler):
             self.model = importlib.import_module(spec)
         return self.model
 
-    def get_node_title(self, default='Rattail'):
-        """
-        Returns the configured title for the local app node.
-        """
-        title = self.config.get('rattail', 'node_title')
-        if title:
-            return title
-        return self.get_title(default=default)
+    def get_version(self, **kwargs):
+        """ """
 
-    def get_version(self):
-        """
-        Returns the current app version.
-        """
-        pkg = self.config.app_package()
-        mod = importlib.import_module(pkg)
-        return mod.__version__
+        # try standard logic first if kwargs specified
+        if kwargs:
+            version = super().get_version(**kwargs)
+            if version:
+                return version
+
+        # if config has defined the (legacy) rattail.app_package
+        # setting, use that (this is hacky but it works..)
+        pkg = self.config.app_package(default='BOGUS', warn=False)
+        if pkg != 'BOGUS':
+            warnings.warn("config has a `rattail.app_package` setting, "
+                          "but that is deprecated.  please set "
+                          "`rattail.app_dist` (or nothing) instead.",
+                          DeprecationWarning)
+            mod = importlib.import_module(pkg)
+            return mod.__version__
+
+        # otherwise use upstream logic
+        return super().get_version(**kwargs)
 
     def get_class_prefix(self, default=None):
         """
@@ -358,60 +365,46 @@ class AppHandler(WuttaAppHandler):
                                     tzinfo=True if tzinfo is None else tzinfo)
             return dt
 
-    def save_setting(self, session, name, value, typ=None,
-                     force_create=False, **kwargs):
+    def save_setting(
+            self,
+            session,
+            name,
+            value,
+            typ=None,
+            **kwargs,
+    ):
         """
-        Save a setting value to the DB.
+        Save a :term:`config setting` value to the DB.
 
-        :param session: Current DB session.
-
-        :param name: Name of the setting to save.
-
-        :param value: Value to be saved for the setting.
+        Rattail overrides
+        :meth:`wuttjamaican:wuttjamaican.app.AppHandler.save_setting()`
+        to add the ``typ`` param.
 
         :param typ: Most values are treated as simple strings, but if
            you specify ``'utctime'`` here, then the ``value`` is
            assumed to be a UTC-based ``datetime`` object, and the
            final setting value will be formatted appropriately.
-
-        :param force_create: If ``False`` (the default) then logic
-           will first try to locate an existing setting of the given
-           name, and update it if found, or create if not.  But if
-           this param is ``True`` then logic will only try to create a
-           new record, and not bother checking to see if it exists.
         """
-        model = self.model
 
+        # maybe coerce value
         if typ == 'utctime':
             if value:
                 value = value.strftime(self.setting_utctime_format)
             else:
                 value = None
 
-        # create or update the setting
-        setting = None
-        if not force_create:
-            setting = session.get(model.Setting, name)
-        if not setting:
-            setting = model.Setting(name=name)
-            session.add(setting)
-        setting.value = value
+        # upstream save
+        super().save_setting(session, name, value, **kwargs)
 
         # invalidate beaker config cache for this setting, if applicable
         self.config.beaker_invalidate_setting(name)
 
-    def delete_setting(self, session, name, **kwargs):
-        """
-        Delete the given setting from the DB.
+    # TODO: remove this
+    def delete_setting(self, session, name):
+        """ """
 
-        :param session: Current DB session.
-
-        :param name: Name of the setting to delete.
-        """
-        model = self.model
-        setting = session.get(model.Setting, name)
-        if setting:
-            session.delete(setting)
+        # upstream delete
+        super().delete_setting(session, name)
 
         # invalidate beaker config cache for this setting, if applicable
         self.config.beaker_invalidate_setting(name)
@@ -1375,16 +1368,6 @@ class AppHandler(WuttaAppHandler):
 
         return prettify(field)
 
-    def get_session(self, obj):
-        """
-        Returns the SQLAlchemy session with which the given object is
-        associated.  Simple convenience wrapper around
-        :func:`sqlalchemy:sqlalchemy.orm.object_session()`.
-        """
-        from sqlalchemy import orm
-
-        return orm.object_session(obj)
-
     def get_runas_user(self, session=None, username=None, required=False):
         """
         Returns a proper :class:`~rattail.db.model.users.User` object
@@ -2008,43 +1991,15 @@ class AppHandler(WuttaAppHandler):
         # but if fallback specified, use that
         return fallback
 
-    def render_duration(self, delta=None, hours=None, seconds=None, **kwargs):
+    def render_duration(self, **kwargs):
         """
         Render a time duration for human eyes, e.g. "1:30" for 1.5 hours.
 
-        Note that you must specify either ``delta``, ``hours`` or ``seconds``.
-
-        :param delta: If specified, should be a ``datetime.timedelta``
-           object representing the duration.
-
-        :param hours: If specified, should be the number of hours
-           elapsed for the duration, as decimal.
-
-        :param seconds: If specified, should be the number of seconds
-           elapsed for the duration, as integer.
-
-        :returns: Duration rendered as human-friendly string.  Note
-           that if all params are empty, this will return an empty
-           string.
+        This is a convenience wrapper around
+        :func:`rattail.util.render_duration()`; please see that for
+        more info.
         """
-        if delta is None and hours is None and seconds is None:
-            return ""
-
-        if delta is None:
-            if seconds is None:
-                seconds = int(hours * 60 * 60)
-            delta = datetime.timedelta(seconds=seconds)
-
-        # calculate minutes for the 'seconds' portion of delta.  note
-        # that we must ensure it is correctly rounded
-        minutes = delta.seconds / 60
-        minutes = decimal.Decimal(str(minutes)).quantize(decimal.Decimal('1'))
-
-        # now add in minutes for the 'days' portion of delta
-        minutes += delta.days * 24 * 60
-
-        # calculate/render final string
-        return "{}:{:02d}".format(int(minutes // 60), int(minutes % 60))
+        return render_duration(**kwargs)
 
     def render_percent(self, value, places=2, from_decimal=False,
                        **kwargs):
@@ -2251,14 +2206,6 @@ class RattailProvider(WuttaAppProvider):
     :class:`~wuttjamaican:wuttjamaican.app.AppProvider` and adds the
     following to it:
 
-    .. attribute:: enum
-
-       Reference to the ``enum`` module for the app.
-
-    .. attribute:: model
-
-       Reference to the ``model`` module for the app.
-
     .. attribute:: handlers
 
        Dictionary of "secondary" handlers used by the provider, if
@@ -2267,15 +2214,31 @@ class RattailProvider(WuttaAppProvider):
 
     def __init__(self, config):
         super().__init__(config)
-        self.enum = self.app.enum
-        self.model = self.app.model
         self.handlers = {}
 
+    @property
+    def enum(self):
+        """ """
+        # TODO
+        # warnings.warn("AppProvider.enum is deprecated; "
+        #               "please use AppProvider.app.enum instead",
+        #               DeprecationWarning, stacklevel=2)
+        return self.app.enum
+
+    @property
+    def model(self):
+        """ """
+        # TODO
+        # warnings.warn("AppProvider.model is deprecated; "
+        #               "please use AppProvider.app.model instead",
+        #               DeprecationWarning, stacklevel=2)
+        return self.app.model
+
     def load_object(self, *args, **kwargs):
-        """
-        Convenience method which calls
-        :meth:`AppHandler.load_object()`.
-        """
+        """ """
+        warnings.warn("AppProvider.load_object() is deprecated; "
+                      "please use AppProvider.app.load_object() instead",
+                      DeprecationWarning, stacklevel=2)
         return self.app.load_object(*args, **kwargs)
 
 
