@@ -37,7 +37,7 @@ from wuttjamaican import auth as base
 from rattail.app import MergeMixin
 
 
-class AuthHandler(base.AuthHandler, MergeMixin):
+class RattailAuthHandler(base.AuthHandler, MergeMixin):
     """
     Default :term:`auth handler` for Rattail.
 
@@ -86,22 +86,6 @@ class AuthHandler(base.AuthHandler, MergeMixin):
     # extra methods
     ##############################
 
-    def user_is_admin(self, user, **kwargs):
-        """
-        Check if given user is a member of the "Administrator" role.
-
-        :rtype: bool
-        """
-        if not user:
-            return False
-
-        session = self.app.get_session(user)
-        admin = self.get_role_administrator(session)
-        if admin in user.roles:
-            return True
-
-        return False
-
     def authenticate_user_token(self, session, token):
         """
         Authenticate the given user API token string, and if valid,
@@ -324,3 +308,12 @@ class AuthHandler(base.AuthHandler, MergeMixin):
                 return True
             return False
         return True
+
+
+class AuthHandler(RattailAuthHandler):
+
+    def __init__(self, *args, **kwargs):
+        warnings.warn("rattail.auth.AuthHandler is deprecated; "
+                      "please use RattailAuthHandler instead",
+                      DeprecationWarning, stacklevel=2)
+        super().__init__(*args, **kwargs)
