@@ -26,17 +26,14 @@ Customer Order Batch Handler
 Please note this is different from the Customer Order Handler.
 """
 
-from __future__ import unicode_literals, absolute_import, division
-
 import re
 import decimal
+from collections import OrderedDict
 
-import six
 import sqlalchemy as sa
 
 from rattail.db import model
 from rattail.batch import BatchHandler
-from rattail.util import OrderedDict
 from rattail.time import localtime
 
 
@@ -177,7 +174,7 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         # nb. person is always required
         if customer and not person:
-            person = clientele.get_person(customer)
+            person = self.app.get_person(customer)
         if not person:
             raise ValueError("Must specify a person")
 
@@ -238,11 +235,11 @@ class CustomerOrderBatchHandler(BatchHandler):
         """
         contact = self.get_contact(batch)
         if contact:
-            return six.text_type(contact)
+            return str(contact)
 
         pending = batch.pending_customer
         if pending:
-            return six.text_type(pending)
+            return str(pending)
 
     def get_contact_phones(self, batch):
         """
@@ -515,7 +512,7 @@ class CustomerOrderBatchHandler(BatchHandler):
         vendor = product.cost.vendor if product.cost else None
         info = {
             'uuid': product.uuid,
-            'upc': six.text_type(product.upc),
+            'upc': str(product.upc),
             'item_id': product.item_id,
             'scancode': product.scancode,
             'upc_pretty': product.upc.pretty(),
@@ -547,7 +544,7 @@ class CustomerOrderBatchHandler(BatchHandler):
                 sale_ends = sale_price.ends
                 if sale_ends:
                     sale_ends = localtime(self.config, sale_ends, from_utc=True).date()
-                    info['sale_ends'] = six.text_type(sale_ends)
+                    info['sale_ends'] = str(sale_ends)
                     info['sale_ends_display'] = self.app.render_date(sale_ends)
 
         case_price = None
@@ -559,10 +556,10 @@ class CustomerOrderBatchHandler(BatchHandler):
                 unit_price = sale_price.price
             case_price = (case_size or 1) * unit_price
             case_price = case_price.quantize(decimal.Decimal('0.01'))
-        info['case_price'] = six.text_type(case_price) if case_price is not None else None
+        info['case_price'] = str(case_price) if case_price is not None else None
         info['case_price_display'] = self.app.render_currency(case_price)
 
-        key = self.config.product_key()
+        key = self.app.get_product_key_field()
         if key == 'upc':
             info['key'] = info['upc_pretty']
         else:
@@ -605,7 +602,8 @@ class CustomerOrderBatchHandler(BatchHandler):
                        if choice['key'] != self.enum.UNIT_OF_MEASURE_CASE]
         if not self.allow_unit_orders():
             choices = [choice for choice in choices
-                       if choice['key'] != self.enum.UNIT_OF_MEASURE_EACH]
+                       if choice['key'] not in (self.enum.UNIT_OF_MEASURE_EACH,
+                                                self.enum.UNIT_OF_MEASURE_POUND)]
         return choices
 
     def why_not_add_product(self, product, batch):
@@ -757,7 +755,7 @@ class CustomerOrderBatchHandler(BatchHandler):
             row.product_upc = product.upc
             row.product_item_id = product.item_id
             row.product_scancode = product.scancode
-            row.product_brand = six.text_type(product.brand or "")
+            row.product_brand = str(product.brand or "")
             row.product_description = product.description
             row.product_size = product.size
             row.product_weighed = product.weighed
@@ -823,7 +821,7 @@ class CustomerOrderBatchHandler(BatchHandler):
         row.product_upc = pending.upc
         row.product_item_id = pending.item_id
         row.product_scancode = pending.scancode
-        row.product_brand = six.text_type(pending.brand or pending.brand_name or '')
+        row.product_brand = str(pending.brand or pending.brand_name or '')
         row.product_description = pending.description
         row.product_size = pending.size
         # TODO: is this even important?  pending does not have it
@@ -848,6 +846,7 @@ class CustomerOrderBatchHandler(BatchHandler):
         row.unit_price = row.unit_regular_price
         row.unit_sale_price = None
         row.sale_ends = None
+        row.special_order = True
 
     def remove_row(self, row):
         batch = row.batch
@@ -881,6 +880,10 @@ class CustomerOrderBatchHandler(BatchHandler):
                 row.pending_product = None
                 session.delete(pending)
 
+    def why_not_execute(self, batch, **kwargs):
+        if not self.get_contact(batch) and not batch.pending_customer:
+            return "Must assign a customer for the order"
+
     def execute(self, batch, user=None, progress=None, **kwargs):
         """
         Default behavior here will create and return a new rattail
@@ -888,7 +891,7 @@ class CustomerOrderBatchHandler(BatchHandler):
         customer record.  Override as needed.
         """
         rows = batch.active_rows()
-        order = self.make_new_order(batch, user=user, progress=progress, **kwargs)
+        order = self.make_new_order(batch, rows, user=user, progress=progress, **kwargs)
         self.update_contact_info(batch, user)
         self.mark_pending_things_ready(batch, rows)
         return order
@@ -941,7 +944,7 @@ class CustomerOrderBatchHandler(BatchHandler):
             'email_address': batch.email_address,
         })
 
-    def make_new_order(self, batch, user=None, progress=None, **kwargs):
+    def make_new_order(self, batch, rows, user=None, progress=None, **kwargs):
         """
         Create and return a new rattail Customer Order based on the
         batch contents.
@@ -959,10 +962,14 @@ class CustomerOrderBatchHandler(BatchHandler):
 
         order = model.CustomerOrder()
         order.created_by = user
-        order.status_code = self.enum.CUSTORDER_STATUS_ORDERED
+        order.status_code = self.enum.CUSTORDER_STATUS_INITIATED
         for field in batch_fields:
             setattr(order, field, getattr(batch, field))
 
+        new_order_id = kwargs.get('new_order_id')
+        if new_order_id:
+            order.id = new_order_id
+
         session = self.app.get_session(batch)
         session.add(order)
         session.flush()
@@ -971,6 +978,8 @@ class CustomerOrderBatchHandler(BatchHandler):
             'product',
             'pending_product_uuid',
             'product_upc',
+            'product_item_id',
+            'product_scancode',
             'product_brand',
             'product_description',
             'product_size',
@@ -984,6 +993,7 @@ class CustomerOrderBatchHandler(BatchHandler):
             'unit_price',
             'discount_percent',
             'total_price',
+            'special_order',
             'price_needs_confirmation',
             'paid_amount',
             'payment_transaction_number',
@@ -1001,7 +1011,7 @@ class CustomerOrderBatchHandler(BatchHandler):
             # set initial status and attach events
             self.set_initial_item_status(item, user)
 
-        self.progress_loop(convert, batch.active_rows(), progress,
+        self.progress_loop(convert, rows, progress,
                            message="Converting batch rows to order items")
 
         session.flush()
@@ -1034,7 +1044,7 @@ class CustomerOrderBatchHandler(BatchHandler):
 
             # then we set "ready" status
             item.status_code = self.enum.CUSTORDER_ITEM_STATUS_READY
-            item.status_text = "everything looks normal"
+            item.status_text = None
 
             item.add_event(self.enum.CUSTORDER_ITEM_EVENT_READY, user,
                            note=item.status_text)
