@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,14 +24,10 @@
 Handheld batch handler
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import csv
 import decimal
 
-import six
-
-from rattail.db import api, model
+from rattail.db import model
 from rattail.batch import BatchHandler
 from rattail.time import make_utc
 from rattail.wince import parse_batch_file as parse_wince_file
@@ -81,8 +77,7 @@ class HandheldBatchHandler(BatchHandler):
         Parse a RattailCE (binary or CSV) file to generate initial rows.
         """
         data = []
-        filemode = 'rt' if six.PY3 else 'rb'
-        with open(path, filemode) as f:
+        with open(path, 'rt') as f:
             line = f.readline()
 
         if '\x00' in line:      # raw binary file from RattailCE app
@@ -113,7 +108,7 @@ class HandheldBatchHandler(BatchHandler):
             if '\t' in line:
                 delimiter = '\t'
 
-            with open(path, filemode) as f:
+            with open(path, 'rt') as f:
                 reader = csv.DictReader(f, delimiter=delimiter)
                 entries = list(reader)
 
@@ -160,7 +155,7 @@ class HandheldBatchHandler(BatchHandler):
             return
 
         session = self.app.get_session(row)
-        product = api.get_product_by_upc(session, row.upc)
+        product = self.locate_product_for_entry(session, row.upc)
         if not product:
             row.status_code = row.STATUS_PRODUCT_NOT_FOUND
             return
