@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,6 @@
 Data Batch Handlers
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import os
 import shutil
 import datetime
@@ -33,16 +31,8 @@ import warnings
 import logging
 
 import json
-import six
-from sqlalchemy import orm
 
-from rattail.db import model, api
-from rattail.core import Object
-from rattail.gpc import GPC
 from rattail.barcodes import upce_to_upca
-from rattail.db.cache import cache_model
-from rattail.time import localtime, make_utc
-from rattail.util import progress_loop, load_object
 
 
 log = logging.getLogger(__name__)
@@ -55,6 +45,13 @@ class BatchHandler(object):
     class, therefore it defines the implementation "interface" loosely
     speaking.  Custom batch handlers are welcome to supplement or override this
     as needed, and in fact must do so for certain aspects.
+
+    .. attribute:: pseudo_remove_rows
+
+       Flag indicating that when a row is "removed" it is merely
+       marked as such, but is not deleted outright.  This flag is on
+       by default; if a handler sets it to false then rows will be
+       deleted outright instead.
     """
     populate_batches = False
     populate_with_versioning = True
@@ -65,11 +62,14 @@ class BatchHandler(object):
     execute_with_versioning = True
     delete_with_versioning = True
 
+    # TODO: this should probably default to false
+    pseudo_remove_rows = True
+
     def __init__(self, config, **kwargs):
         self.config = config
         self.app = self.config.get_app()
+        self.model = self.app.model
         self.enum = config.get_enum()
-        self.model = config.get_model()
 
     @property
     def batch_model_class(self):
@@ -112,7 +112,8 @@ class BatchHandler(object):
             return self.execute_with_versioning
         if action == 'delete':
             return self.delete_with_versioning
-        raise NotImplementedError("unknown batch action: {}".format(action))
+        log.warning("unknown batch action: %s", action)
+        return True
 
     def is_mutable(self, batch):
         """
@@ -219,7 +220,7 @@ class BatchHandler(object):
         ``rowcount`` is incremented, and the :meth:`after_add_row()` hook is
         invoked.
         """
-        session = orm.object_session(batch)
+        session = self.app.get_session(batch)
         with session.no_autoflush:
             batch.data_rows.append(row)
             self.refresh_row(row)
@@ -234,6 +235,13 @@ class BatchHandler(object):
         batch, to account for the new row, etc.
         """
 
+    def is_row_deletable(self, row, **kwargs):
+        """
+        General logic to determine if deleting the given row from its
+        batch should be allowed.
+        """
+        return True
+
     def purge_batches(self, session, before=None, before_days=90,
                       dry_run=False, delete_all_data=None,
                       progress=None, **kwargs):
@@ -258,15 +266,17 @@ class BatchHandler(object):
 
         :returns: Integer indicating the number of batches purged.
         """
+        from sqlalchemy import orm
+
         if delete_all_data and dry_run:
             raise ValueError("You can enable (n)either of `dry_run` or "
                              "`delete_all_data` but both cannot be True")
         delete_all_data = not dry_run
 
         if not before:
-            before = localtime(self.config).date() - datetime.timedelta(days=before_days)
+            before = self.app.today() - datetime.timedelta(days=before_days)
             before = datetime.datetime.combine(before, datetime.time(0))
-            before = localtime(self.config, before)
+            before = self.app.localtime(before)
 
         log.info("will purge '%s' batches, executed before %s",
                  self.batch_key, before.date())
@@ -276,7 +286,7 @@ class BatchHandler(object):
                              .options(orm.joinedload(self.batch_model_class.data_rows))\
                              .all()
         log.info("found %s batches to purge", len(old_batches))
-        result = Object()
+        result = self.app.make_object()
         result.purged = 0
 
         def purge(batch, i):
@@ -420,7 +430,7 @@ class BatchHandler(object):
         return True
 
     def progress_loop(self, *args, **kwargs):
-        return progress_loop(*args, **kwargs)
+        return self.app.progress_loop(*args, **kwargs)
 
     def setup_refresh(self, batch, progress=None):
         """
@@ -461,7 +471,7 @@ class BatchHandler(object):
         should *not* override the :meth:`~do_refresh()` method, but callers
         *should* use that one directly.
         """
-        session = orm.object_session(batch)
+        session = self.app.get_session(batch)
         self.setup_refresh(batch, progress=progress)
         if self.repopulate_when_refresh:
             del batch.data_rows[:]
@@ -531,7 +541,7 @@ class BatchHandler(object):
 
         row.item_id = product.item_id
         row.upc = product.upc
-        row.brand_name = six.text_type(product.brand or "")
+        row.brand_name = str(product.brand or "")
         row.description = product.description
         row.size = product.size
 
@@ -586,7 +596,12 @@ class BatchHandler(object):
         *should* use that one directly.
         """
         batch = row.batch
-        row.removed = True
+        if self.pseudo_remove_rows:
+            row.removed = True
+        else:
+            session = self.app.get_session(batch)
+            session.delete(row)
+            session.flush()
         self.refresh_batch_status(batch)
 
     def do_remove_row(self, row):
@@ -661,7 +676,7 @@ class BatchHandler(object):
         """
         batch.complete = False
 
-    def why_not_execute(self, batch):
+    def why_not_execute(self, batch, user=None, **kwargs):
         """
         This method should inspect the given batch and, if there is a reason
         that execution should *not* be allowed for it, the method should return
@@ -728,6 +743,10 @@ class BatchHandler(object):
         *should* override the :meth:`~execute()` method, but callers should
         *not* use that one directly.
         """
+        reason = self.why_not_execute(batch, user=user)
+        if reason:
+            raise RuntimeError(f"Execute not allowed: {reason}")
+
         # make sure we declare who's responsible, if we can
         # TODO: seems like if caller already knows user, they should
         # have already done this.  and probably bad form to do it here
@@ -738,7 +757,7 @@ class BatchHandler(object):
         if not result:
             return False
 
-        batch.executed = make_utc()
+        batch.executed = self.app.make_utc()
         batch.executed_by = user
 
         # record the execution kwargs within batch params, if there
@@ -805,7 +824,7 @@ class BatchHandler(object):
         Handlers may have to override this method if "grouping" or other
         special behavior is needed.
         """
-        now = make_utc()
+        now = self.app.make_utc()
         for batch in batches:
             if not batch.executed:
                 self.execute(batch, progress=progress, **kwargs)
@@ -823,12 +842,12 @@ class BatchHandler(object):
         *should* override the :meth:`~delete()` method, but callers should
         *not* use that one directly.
         """
-        session = orm.object_session(batch)
+        session = self.app.get_session(batch)
 
         if 'delete_all_data' in kwargs:
             warnings.warn("The 'delete_all_data' kwarg is not supported for "
                           "this method; please use 'dry_run' instead",
-                          DeprecationWarning)
+                          DeprecationWarning, stacklevel=2)
         kwargs['delete_all_data'] = not dry_run
 
         self.delete(batch, progress=progress, **kwargs)
@@ -859,7 +878,7 @@ class BatchHandler(object):
         # TODO: in other words i don't even know why this is necessary.  seems
         # to me that one fell swoop should not incur FK errors
         if hasattr(batch, 'data_rows'):
-            session = orm.object_session(batch)
+            session = self.app.get_session(batch)
 
             def delete(row, i):
                 session.delete(row)
@@ -908,10 +927,12 @@ class BatchHandler(object):
         was just created by cloning the old batch.
         """
 
-    def clone(self, oldbatch, created_by, progress=None):
+    def clone(self, oldbatch, created_by, progress=None, **kwargs):
         """
         Clone the given batch as a new batch, and return the new batch.
         """
+        from sqlalchemy import orm
+
         self.setup_clone(oldbatch, progress=progress)
         batch_class = self.batch_model_class
         batch_mapper = orm.class_mapper(batch_class)
@@ -934,14 +955,19 @@ class BatchHandler(object):
             newrow = self.clone_row(oldrow)
             self.add_row(newbatch, newrow)
 
-        self.progress_loop(clone_row, oldbatch.data_rows, progress,
+        self.progress_loop(clone_row, self.get_clonable_rows(oldbatch), progress,
                            message="Cloning data rows for new batch")
 
         self.refresh_batch_status(newbatch)
         self.teardown_clone(newbatch, progress=progress)
         return newbatch
 
+    def get_clonable_rows(self, batch, **kwargs):
+        return batch.data_rows
+
     def clone_row(self, oldrow):
+        from sqlalchemy import orm
+
         row_class = self.batch_model_class.row_class
         row_mapper = orm.class_mapper(row_class)
         newrow = row_class()
@@ -950,15 +976,16 @@ class BatchHandler(object):
                 setattr(newrow, name, getattr(oldrow, name))
         return newrow
 
-    def cache_model(self, session, model, **kwargs):
-        return cache_model(session, model, **kwargs)
+    def cache_model(self, session, model_class, **kwargs):
+        return self.app.cache_model(session, model_class, **kwargs)
 
 
 def get_batch_types(config):
     """
     Returns the list of available batch type keys.
     """
-    model = config.get_model()
+    app = config.get_app()
+    model = app.model
 
     keys = []
     for name in dir(model):
@@ -977,6 +1004,6 @@ def get_batch_types(config):
 def get_batch_handler(config, batch_key, default=None, error=True): # pragma: no cover
     warnings.warn("function is deprecated; please use "
                   "`app.get_batch_handler() instead",
-                  DeprecationWarning)
+                  DeprecationWarning, stacklevel=2)
     app = config.get_app()
     return app.get_batch_handler(batch_key, default=default, error=error)
