@@ -141,7 +141,9 @@ class NewProductBatchHandler(BatchHandler):
                 row.status_code = row.STATUS_MISSING_KEY
                 return
 
-            row.product = self.locate_product_for_entry(session, row.item_entry)
+            row.product = self.locate_product_for_entry(
+                session, row.item_entry,
+                type2_lookup=row.batch.get_param('type2_lookup'))
 
         if row.product:
             row.status_code = row.STATUS_PRODUCT_EXISTS
