@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,6 @@
 Batch utilities
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import inspect
 
 
@@ -40,8 +38,10 @@ def consume_batch_id(session, as_str=False):
     """
     Consumes and returns the next batch ID from PG sequence.
     """
+    import sqlalchemy as sa
+
     sql = "select nextval('batch_id_seq')"
-    batch_id = session.execute(sql).scalar()
+    batch_id = session.execute(sa.text(sql)).scalar()
     if as_str:
         return batch_id_str(batch_id)
     return batch_id
