@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,6 @@
 Email Bounce Handlers
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import os
 import datetime
 import logging
@@ -33,12 +31,9 @@ import warnings
 from email import message_from_string
 from email.utils import parsedate_tz, mktime_tz
 
-import six
-from sqlalchemy import orm
 from flufl.bounce import all_failures
 
 from rattail.core import Object
-from rattail.util import load_object
 
 
 log = logging.getLogger(__name__)
@@ -50,7 +45,7 @@ def get_handler(config, key): # pragma: no cover
     """
     warnings.warn("function is deprecated; please use "
                   "`app.get_bounce_handler() instead",
-                  DeprecationWarning)
+                  DeprecationWarning, stacklevel=2)
     app = config.get_app()
     return app.get_bounce_handler(key)
 
@@ -62,7 +57,7 @@ class Link(Object):
     """
 
 
-class BounceHandler(object):
+class BounceHandler:
     """
     Default implementation for email bounce handlers.
     """
@@ -71,23 +66,21 @@ class BounceHandler(object):
         self.config = config
         self.config_key = config_key
         self.app = config.get_app()
-        self.model = config.get_model()
         self.enum = config.get_enum()
 
+        try:
+            self.model = self.app.model
+        except ImportError:
+            pass
+
     def get_all_failures(self, msg):
         warnings, failures = all_failures(msg)
         if warnings:
-            if six.PY3:
-                warnings = ','.join(sorted(
-                    [e.decode('utf_8') for e in warnings]))
-            else:
-                warnings = ','.join(sorted(warnings))
+            warnings = ','.join(sorted(
+                [e.decode('utf_8') for e in warnings]))
         if failures:
-            if six.PY3:
-                failures = ','.join(sorted(
-                    [e.decode('utf_8') for e in failures]))
-            else:
-                failures = ','.join(sorted(failures))
+            failures = ','.join(sorted(
+                [e.decode('utf_8') for e in failures]))
         return warnings, failures
 
     @property
@@ -175,12 +168,8 @@ class BounceHandler(object):
         msgdir = self.msgdir(bounce)
         os.makedirs(msgdir)
         path = os.path.join(msgdir, 'bounce.eml')
-        if six.PY3:
-            with open(path, 'wt') as f:
-                f.write(msg_body)
-        else:
-            with open(path, 'wb') as f:
-                f.write(msg_body)
+        with open(path, 'wt') as f:
+            f.write(msg_body)
         return path
 
     def make_link(self, **kwargs):
@@ -193,12 +182,12 @@ class BounceHandler(object):
         emails = session.query(model.CustomerEmailAddress).filter_by(address=recipient)
         for email in emails:
             yield self.make_link(type="Rattail Customer",
-                                 title=six.text_type(email.customer),
+                                 title=str(email.customer),
                                  url=url.format(uuid=email.parent_uuid))
 
         url = self.config.require('tailbone', 'url.person')
         emails = session.query(model.PersonEmailAddress).filter_by(address=recipient)
         for email in emails:
             yield self.make_link(type="Rattail Person",
-                                 title=six.text_type(email.person),
+                                 title=str(email.person),
                                  url=url.format(uuid=email.parent_uuid))
