@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,37 @@
 Console Commands
 """
 
-from __future__ import unicode_literals, absolute_import
+from .base import rattail_typer
 
-from .core import main, Command, Subcommand, date_argument, list_argument
-from .importing import ImportSubcommand, ImportFromCSV, ImportFileSubcommand, ExportFileSubcommand
-from .install import InstallSubcommand
+# TODO: is this the best we can do, to register available commands?
+from . import backup
+from . import batch
+from . import bouncer
+from . import checkdb
+from . import cleanup
+from . import clonedb
+from . import datasync
+from . import date_organize
+from . import filemon
+from . import importing
+from . import luigi
+from . import mailmon
+from . import make_appdir
+from . import make_config
+from . import make_user
+from . import make_uuid
+from . import mysql
+from . import postfix
+from . import problems
+from . import products
+from . import projects
+from . import run_n_mail
+from . import runsql
+from . import settings
+from . import telemetry
+from . import upgrade
+from . import versions
+
+# discover more commands
+from .typer import typer_eager_imports
+typer_eager_imports(rattail_typer)
