@@ -85,12 +85,37 @@ class RattailConfig(WuttaConfig):
 
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('appname', 'rattail')
-        defaults = kwargs.setdefault('defaults', {})
         super().__init__(*args, **kwargs)
 
         # this is false, unless/until it becomes true
         self.versioning_has_been_enabled = False
 
+        # configure session if applicable
+        from rattail.db import Session
+        if Session:
+
+            # nb. wutta configures its own Session but we must
+            # configure ours separately (albeit with same engine)
+            if hasattr(self, 'appdb_engine'):
+                Session.configure(bind=self.appdb_engine)
+
+            # TODO: eventually this should not be needed (?)
+            Session.configure(rattail_config=self)
+
+    @property
+    def rattail_engines(self):
+        warnings.warn("config.rattail_engines is deprecated; "
+                      "please use config.appdb_engines instead",
+                      DeprecationWarning, stacklevel=2)
+        return self.appdb_engines
+
+    @property
+    def rattail_engine(self):
+        warnings.warn("config.rattail_engine is deprecated; "
+                      "please use config.appdb_engine instead",
+                      DeprecationWarning, stacklevel=2)
+        return self.appdb_engine
+
     @property
     def prioritized_files(self):
         """
@@ -327,31 +352,29 @@ class RattailConfig(WuttaConfig):
                       DeprecationWarning, stacklevel=2)
         return self.get_app().get_product_key_label(field=key)
 
-    def app_package(self, default=None):
-        """
-        Returns the name of Python package for the top-level app.
-        """
+    def app_package(self, default=None, warn=True):
+        """ """
+        if warn:
+            warnings.warn("config.app_package() method is deprecated; "
+                          "please use app.get_distribution() instead",
+                          DeprecationWarning, stacklevel=2)
         if not default:
             return self.require('rattail', 'app_package')
         return self.get('rattail', 'app_package', default=default)
 
     def app_title(self, **kwargs):
-        """
-        DEPRECATED; use :meth:`rattail.app.AppHandler.get_title()`
-        instead.
-        """
-        # TODO: should put a deprecation warning here, but it could
-        # make things noisy for a while and i'm not ready for that
+        """ """
+        warnings.warn("config.app_title() method is deprecated; "
+                      "please use app.get_title() instead",
+                      DeprecationWarning, stacklevel=2)
         app = self.get_app()
         return app.get_title(**kwargs)
 
     def node_title(self, **kwargs):
-        """
-        DEPRECATED; use
-        :meth:`rattail.app.AppHandler.get_node_title()` instead.
-        """
-        # TODO: should put a deprecation warning here, but it could
-        # make things noisy for a while and i'm not ready for that
+        """ """
+        warnings.warn("config.node_title() method is deprecated; "
+                      "please use app.get_node_title() instead",
+                      DeprecationWarning, stacklevel=2)
         app = self.get_app()
         return app.get_node_title(**kwargs)
 
@@ -527,13 +550,146 @@ class RattailConfig(WuttaConfig):
 
 class ConfigExtension(WuttaConfigExtension):
     """
-    Base class for all config extensions.
+    DEPRECATED - use
+    :class:`wuttjamaican:wuttjamaican.conf.WuttaConfigExtension`
+    instead.
 
-    This is just a compatibility wrapper around
-    :class:`wuttjamaican:wuttjamaican.conf.WuttaConfigExtension`; new
-    code should probably use that directly.
+    See also :class:`RattailConfigExtension`.
     """
 
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        warnings.warn(f"rattail.config.ConfigExtension (for '{self.key}') is deprecated; "
+                      "please use wuttjamaican.conf.WuttaConfigExtension instead",
+                      DeprecationWarning, stacklevel=2)
+
+
+class RattailConfigExtension(WuttaConfigExtension):
+    """
+    The :term:`config extension` for Rattail.
+
+    This is a subclass of
+    :class:`wuttjamaican:wuttjamaican.conf.WuttaConfigExtension`.
+
+    This primarily exists to add default config settings for sake of
+    the various importers etc. contained in the ``rattail`` package.
+
+    .. note::
+
+       If you are adding a new config extension, **do not subclass
+       this**, but rather subclass
+       :class:`wuttjamaican:wuttjamaican.conf.WuttaConfigExtension`
+       directly.
+    """
+    key = 'rattail.db'
+
+    def configure(self, config):
+        """ """
+
+        # rattail export-csv
+        config.setdefault('rattail.importing', 'to_csv.from_rattail.export.default_handler',
+                          'rattail.importing.exporters:FromRattailToCSV')
+        config.setdefault('rattail.importing', 'to_csv.from_rattail.export.default_cmd',
+                          'rattail export-csv')
+        config.setdefault('rattail.importing', 'to_csv.from_rattail.export.legacy_handler_setting',
+                          'rattail.exporting, csv.handler')
+
+        # rattail export-rattail
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.export.default_handler',
+                          'rattail.importing.rattail:FromRattailToRattailExport')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.export.default_cmd',
+                          'rattail export-rattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.export.legacy_handler_setting',
+                          'rattail.exporting, rattail.handler')
+
+        # rattail import-csv
+        config.setdefault('rattail.importing', 'to_rattail.from_csv.import.default_handler',
+                          'rattail.importing.csv:FromCSVToRattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_csv.import.default_cmd',
+                          'rattail import-csv')
+        config.setdefault('rattail.importing', 'to_rattail.from_csv.import.legacy_handler_setting',
+                          'rattail.importing, csv.handler')
+
+        # rattail import-ifps
+        config.setdefault('rattail.importing', 'to_rattail.from_ifps.import.default_handler',
+                          'rattail.importing.ifps:FromIFPSToRattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_ifps.import.default_cmd',
+                          'rattail import-ifps')
+        config.setdefault('rattail.importing', 'to_rattail.from_ifps.import.legacy_handler_setting',
+                          'rattail.importing, ifps.handler')
+
+        # rattail import-rattail
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.import.default_handler',
+                          'rattail.importing.rattail:FromRattailToRattailImport')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.import.default_cmd',
+                          'rattail import-rattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail.import.legacy_handler_setting',
+                          'rattail.importing, rattail.handler')
+
+        # rattail import-rattail-bulk
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail_bulk.import.default_handler',
+                          'rattail.importing.rattail_bulk:BulkFromRattailToRattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail_bulk.import.default_cmd',
+                          'rattail import-rattail-bulk')
+        config.setdefault('rattail.importing', 'to_rattail.from_rattail_bulk.import.legacy_handler_setting',
+                          'rattail.importing, rattail_bulk.handler')
+
+        # rattail import-sample
+        config.setdefault('rattail.importing', 'to_rattail.from_sample.import.default_handler',
+                          'rattail.importing.sample:FromSampleToRattail')
+        config.setdefault('rattail.importing', 'to_rattail.from_sample.import.default_cmd',
+                          'rattail import-sample')
+        config.setdefault('rattail.importing', 'to_rattail.from_sample.import.legacy_handler_setting',
+                          'rattail.importing, sample.handler')
+
+        # rattail import-versions
+        config.setdefault('rattail.importing', 'to_rattail_versions.from_rattail.import.default_handler',
+                          'rattail.importing.versions:FromRattailToRattailVersions')
+        config.setdefault('rattail.importing', 'to_rattail_versions.from_rattail.import.default_cmd',
+                          'rattail import-versions')
+        config.setdefault('rattail.importing', 'to_rattail_versions.from_rattail.import.legacy_handler_setting',
+                          'rattail.importing, versions.handler')
+
+        # trainwreck export-trainwreck
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.export.default_handler',
+                          'rattail.trainwreck.importing.trainwreck:FromTrainwreckToTrainwreckExport')
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.export.default_cmd',
+                          'trainwreck export-trainwreck')
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.export.legacy_handler_setting',
+                          'trainwreck.exporting, trainwreck.handler')
+
+        # trainwreck import-self
+        config.setdefault('rattail.importing', 'to_trainwreck.from_self.import.default_cmd',
+                          'trainwreck import-self')
+
+        # trainwreck import-trainwreck
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.import.default_handler',
+                          'rattail.trainwreck.importing.trainwreck:FromTrainwreckToTrainwreckImport')
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.import.default_cmd',
+                          'trainwreck import-trainwreck')
+        config.setdefault('rattail.importing', 'to_trainwreck.from_trainwreck.import.legacy_handler_setting',
+                          'trainwreck.importing, trainwreck.handler')
+
+        # nb. cannot fetch DB settings during init
+        appdir = config.appdir(require=False, usedb=False)
+        if not appdir:
+            appdir = os.path.join(sys.prefix, 'app')
+
+        poser = config.get('rattail', 'poser', usedb=False,
+                           default=os.path.join(appdir, 'poser'))
+
+        # add poser to path if it exists
+        if os.path.isdir(poser) and poser not in sys.path:
+            sys.path.append(poser)
+
+    def startup(self, config):
+        from rattail.db import Session
+        if Session:
+
+            # TODO: this should be removed, it sets 'record changes' globally
+            from rattail.db.config import configure_session
+            configure_session(config, Session)
+
 
 def rattail_default_files(appname):
     """
@@ -602,9 +758,8 @@ def make_config(
 
     # make config object
     config = wutta_make_config(files=files, plus_files=plus_files, **kwargs)
-    log.debug("config files were: %s", config.files_read)
 
-    if config.getbool('rattail', 'suppress_psycopg2_wheel_warning', usedb=False):
+    if config.get_bool('rattail.suppress_psycopg2_wheel_warning', usedb=False):
         # TODO: revisit this, does it require action from us?
         # suppress this warning about psycopg2 wheel; not sure what it means yet
         # exactly but it's causing frequent noise for us...
@@ -624,8 +779,25 @@ def make_config(
         from rattail.db.config import configure_versioning
         configure_versioning(config)
 
+        # TODO: holy crap why is this needed?  without it, the first time
+        # a trainwreck query happens, InvalidRequestError is raised.  but
+        # this is clearly a trainwreck query, and yet no error if we do it
+        # this early in the startup sequence?!  i have no idea what is
+        # going on here but at least this "works" - fingers crossed..
+        if getattr(config, 'trainwreck_engines', None):
+            app = config.get_app()
+            trainwreck = app.get_trainwreck_handler()
+            try:
+                trainwreck_model = trainwreck.get_model()
+            except WuttaConfigurationError:
+                pass
+            else:
+                trainwreck_session = trainwreck.make_session()
+                trainwreck_session.query(trainwreck_model.Transaction).first()
+                trainwreck_session.close()
+
     # maybe set "future" behavior for SQLAlchemy
-    if config.getbool('rattail.db', 'sqlalchemy_future_mode', usedb=False):
+    if config.get_bool('rattail.db.sqlalchemy_future_mode', usedb=False):
         from rattail.db import Session
         if Session:
             Session.configure(future=True)
