@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,17 +24,13 @@
 Vendor catalog parser for Dutch Valley
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import re
 import datetime
 import decimal
 import logging
 
-import six
 import xlrd
 
-from rattail.db import model
 from rattail.vendors.catalogs import CatalogParser
 
 
@@ -66,13 +62,13 @@ class DutchValleyCatalogParser(CatalogParser):
             # Product code is in first column...but so is a lot of other
             # garbage.  Use strict regex to find good rows.
             code = sheet.cell_value(r, 0)
-            if not isinstance(code, six.string_types):
+            if not isinstance(code, str):
                 continue
             if not code_pattern.match(code):
                 continue
             code = code.replace(' ', '')
             
-            row = model.VendorCatalogBatchRow()
+            row = self.make_row()
             row.vendor_code = code
 
             # Try to parse unit and/or case size from description.
@@ -89,13 +85,13 @@ class DutchValleyCatalogParser(CatalogParser):
 
             case_cost = sheet.cell_value(r, 2)
             try:
-                row.case_cost = decimal.Decimal(six.text_type(case_cost))
+                row.case_cost = decimal.Decimal(str(case_cost))
             except ValueError:
                 log.warning("bad case cost at row {0}: {1}".format(r + 1, repr(case_cost)))
 
             unit_cost = sheet.cell_value(r, 4)
             try:
-                row.unit_cost = decimal.Decimal(six.text_type(unit_cost))
+                row.unit_cost = decimal.Decimal(str(unit_cost))
             except ValueError:
                 log.warning("bad unit cost at row {0}: {1}".format(r + 1, repr(unit_cost)))
 
