@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2021 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,13 +24,9 @@
 Vendor invoice parser for Albert's Organics
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import csv
 import datetime
 
-import six
-
 from rattail.db import model
 from rattail.gpc import GPC
 from rattail.vendors.invoices import InvoiceParser
@@ -45,20 +41,14 @@ class AlbertsInvoiceParser(InvoiceParser):
     vendor_key = 'alberts'
 
     def parse_invoice_date(self, path):
-        if six.PY3:
-            csv_file = open(path, 'rt')
-        else: # PY2
-            csv_file = open(path, 'rb')
+        csv_file = open(path, 'rt')
         reader = csv.DictReader(csv_file)
         data = next(reader)
         csv_file.close()
         return datetime.datetime.strptime(data['Invoice Date'], '%m/%d/%Y').date()
 
     def parse_rows(self, path):
-        if six.PY3:
-            csv_file = open(path, 'rt')
-        else: # PY2
-            csv_file = open(path, 'rb')
+        csv_file = open(path, 'rt')
 
         reader = csv.DictReader(csv_file)
         for data in reader:
