@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,13 +24,10 @@
 Vendor invoice parser for KeHE Distributors
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import re
 import csv
 import datetime
-
-import six
+import decimal
 
 from rattail.vendors.invoices import InvoiceParser
 
@@ -59,6 +56,7 @@ class KeheInvoiceParser(InvoiceParser):
     vendor_key = 'kehe'
 
     pack_size_pattern = re.compile('^(?P<case_quantity>\d+)/(?P<size>\d*\.\d+ \w\w)$')
+    bulk_size_pattern = re.compile('^(?P<case_quantity>\d+\.\d+) LB$')
 
     def detect_version(self, path):
         """
@@ -70,12 +68,8 @@ class KeheInvoiceParser(InvoiceParser):
 
         delimiter = str('\t') if '\t' in line else ','
 
-        if six.PY3:
-            csv_file = open(path, 'rt')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
-        else: # PY2
-            csv_file = open(path, 'rb')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
+        csv_file = open(path, 'rt')
+        reader = csv.DictReader(csv_file, delimiter=delimiter)
         data = next(reader)
         csv_file.close()
 
@@ -89,18 +83,27 @@ class KeheInvoiceParser(InvoiceParser):
                 return 3
             return 1
 
+    def parse_invoice_number(self, path):
+
+        # first find out which version of file we have
+        version = self.detect_version(path)
+
+        delimiter = str('\t') if version in (1, 3) else ','
+        csv_file = open(path, 'rt')
+        reader = csv.DictReader(csv_file, delimiter=delimiter)
+        data = next(reader)
+        csv_file.close()
+
+        return data['InvoiceNumber']
+
     def parse_invoice_date(self, path):
 
         # first find out which version of file we have
         version = self.detect_version(path)
 
         delimiter = str('\t') if version in (1, 3) else ','
-        if six.PY3:
-            csv_file = open(path, 'rt')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
-        else: # PY2
-            csv_file = open(path, 'rb')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
+        csv_file = open(path, 'rt')
+        reader = csv.DictReader(csv_file, delimiter=delimiter)
         data = next(reader)
         csv_file.close()
 
@@ -149,16 +152,13 @@ class KeheInvoiceParser(InvoiceParser):
                 'order_quantity': 'OrderQuatity',
             })
 
-        if six.PY3:
-            csv_file = open(path, 'rt')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
-        else: # PY2
-            csv_file = open(path, 'rb')
-            reader = csv.DictReader(csv_file, delimiter=delimiter)
+        csv_file = open(path, 'rt')
+        reader = csv.DictReader(csv_file, delimiter=delimiter)
 
-        for data in reader:
+        for i, data in enumerate(reader, 1):
 
             row = self.make_row()
+            row.line_number = i
             row.item_entry = data[fields['upc']]
             row.upc = self.app.make_gpc(row.item_entry)
             row.vendor_code = data[fields['ship_item']]
@@ -166,9 +166,15 @@ class KeheInvoiceParser(InvoiceParser):
             row.description = data[fields['description']]
             row.ordered_units = self.int_(data[fields['order_quantity']])
             row.shipped_units = self.int_(data[fields['ship_quantity']])
-            row.unit_cost = self.decimal(data[fields['net_each']])
             row.total_cost = self.decimal(data[fields['net_billable']])
 
+            # nb. unit cost ideally would come from NetEach column,
+            # but that is rounded to 2 places so not always accurate
+            if row.total_cost and row.shipped_units:
+                row.unit_cost = row.total_cost / row.shipped_units
+            else:
+                row.unit_cost = self.decimal(data[fields['net_each']])
+
             # Case quantity may be embedded in size string.
             row.size = data[fields['pack_size']]
             row.case_quantity = 1
@@ -177,6 +183,16 @@ class KeheInvoiceParser(InvoiceParser):
                 row.case_quantity = int(match.group('case_quantity'))
                 row.size = match.group('size')
 
+                if row.case_quantity == 1:
+                    match = self.bulk_size_pattern.match(row.size)
+                    if match:
+                        row.case_quantity = decimal.Decimal(match.group('case_quantity'))
+                        row.unit_cost /= row.case_quantity
+                        row.ordered_cases = row.ordered_units
+                        row.ordered_units = None
+                        row.shipped_cases = row.shipped_units
+                        row.shipped_units = None
+
             # attach original raw data to the row we're returning; caller
             # can use as needed, or ignore
             row._raw_data = data
