@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,16 +24,12 @@
 Vendor invoice parser for United Natural Foods (UNFI)
 """
 
-from __future__ import unicode_literals, absolute_import
-
+import csv
 import datetime
 
-import six
-
 from rattail.db import model
 from rattail.gpc import GPC
 from rattail.vendors.invoices import InvoiceParser
-from rattail.csvutil import UnicodeDictReader, UnicodeReader
 
 
 class UnfiInvoiceParser(InvoiceParser):
@@ -44,29 +40,32 @@ class UnfiInvoiceParser(InvoiceParser):
     display = "United Natural Foods (UNFI)"
     vendor_key = 'unfi'
 
+    def open_csv_file(self, path):
+        return open(path, 'rt', encoding='latin_1')
+
+    def parse_invoice_number(self, path):
+        csv_file = self.open_csv_file(path)
+        reader = csv.DictReader(csv_file)
+        data = next(reader)
+        csv_file.close()
+        return data['InvNum']
+
     def parse_invoice_date(self, data_path):
-        if six.PY3:
-            csv_file = open(data_path, 'rt', encoding='latin_1')
-            reader = UnicodeDictReader(csv_file)
-        else: # PY2
-            csv_file = open(data_path, 'rb')
-            reader = UnicodeDictReader(csv_file, encoding='latin_1')
+        csv_file = self.open_csv_file(data_path)
+        reader = csv.DictReader(csv_file)
         data = next(reader)
         csv_file.close()
         return datetime.datetime.strptime(data['InvoiceDate'], '%m/%d/%Y').date()
 
     def parse_rows(self, data_path):
-        if six.PY3:
-            csv_file = open(data_path, 'rt', encoding='latin_1')
-        else: # PY2
-            csv_file = open(data_path, 'rb')
+        csv_file = self.open_csv_file(data_path)
 
         # TODO: The following logic is largely copied from the Scan Genius
         # order parser (in `rattail_livnat.scangenius`).  I wonder if we
         # can abstract it into a generic yet custom CSV parser...?
 
         # We want to ignore the header section here.  However the use of
-        # UnicodeDictReader below requires iteration, and yet also expects
+        # DictReader below requires iteration, and yet also expects
         # the "beginning" of the file to contain the fieldnames.  To force
         # this all to play nicely, we calculate our offset and then seek to
         # it explicitly.  It's probably likely that there is a better way,
@@ -74,10 +73,7 @@ class UnfiInvoiceParser(InvoiceParser):
         offset = self.find_details_offset(csv_file)
         csv_file.seek(offset)
 
-        if six.PY3:
-            reader = UnicodeDictReader(csv_file)
-        else: # PY2
-            reader = UnicodeDictReader(csv_file, encoding='latin_1')
+        reader = csv.DictReader(csv_file)
         for data in reader:
 
             # Only consider 'Detail' rows; this check is mostly for the
@@ -96,9 +92,17 @@ class UnfiInvoiceParser(InvoiceParser):
             row.ordered_cases = int(self.decimal(data['QuantityOrdered']))
             row.shipped_cases = self.int_(data['QuantityShipped'])
             row.out_of_stock = True if data['Status'] == 'OUT' else False
-            row.unit_cost = self.decimal(data['NetPricePerUnit'])
             row.total_cost = self.decimal(data['ExtendedPrice'])
-            row.case_cost = row.unit_cost * row.case_quantity
+
+            # nb. UFNI total cost (ExtendedPrice) may reflect a
+            # discount, whereas unit cost (NetPricePerUnit) does not.
+            # so only use latter if former not available.
+            if row.total_cost:
+                row.case_cost = row.total_cost / row.shipped_cases
+                row.unit_cost = row.case_cost / row.case_quantity
+            else:
+                row.unit_cost = self.decimal(data['NetPricePerUnit'])
+                row.case_cost = row.unit_cost * row.case_quantity
 
             yield row
 
