@@ -1,8 +1,8 @@
-# -*- coding: utf-8 -*-
+# -*- coding: utf-8; -*-
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2017 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,9 +24,9 @@
 Core Stuff
 """
 
-from __future__ import unicode_literals, absolute_import
+import warnings
 
-from uuid import uuid1
+from wuttjamaican.util import make_uuid
 
 
 # Arbitrary object used to explicitly denote an unspecified value, e.g. for
@@ -62,9 +62,10 @@ class Object(object):
 
 def get_uuid():
     """
-    Generate a universally-unique identifier.
-
-    :returns: A 32-character hex string.
+    DEPRECATED; use
+    :func:`~wuttjamaican:wuttjamaican.util.make_uuid()` instead.
     """
-
-    return uuid1().hex
+    warnings.warn("rattail.core.get_uuid() is deprecated; "
+                  "please use wuttjamaican.util.make_uuid() instead",
+                  DeprecationWarning, stacklevel=2)
+    return make_uuid()
