@@ -26,8 +26,6 @@ Customer Orders Handler
 Please note this is different from the Customer Order Batch Handler.
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import decimal
 
 from rattail.app import GenericHandler
@@ -43,6 +41,11 @@ class CustomerOrderHandler(GenericHandler):
         Returns default item discount available.  If product is given,
         the default may be specific to its department etc.
         """
+        if product:
+            department = product.department
+            if department and department.default_custorder_discount is not None:
+                return department.default_custorder_discount
+
         discount = self.config.get('rattail.custorders',
                                    'default_item_discount')
         if discount:
@@ -104,3 +107,41 @@ class CustomerOrderHandler(GenericHandler):
 
             item.add_event(self.enum.CUSTORDER_ITEM_EVENT_PRODUCT_RESOLVED,
                            user)
+
+    def mark_received(self, order_items, user, **kwargs):
+        """
+        Mark the given set of customer order items as having been
+        received (i.e. from the vendor).
+        """
+        model = self.model
+
+        event_kw = {
+            'type_code': self.enum.CUSTORDER_ITEM_EVENT_RECEIVED,
+            'user': user,
+            'note': kwargs.get('note'),
+        }
+
+        for item in order_items:
+            item.status_code = self.enum.CUSTORDER_ITEM_STATUS_RECEIVED
+            item.status_text = None
+            item.events.append(model.CustomerOrderItemEvent(**event_kw))
+
+    def add_note(self, item, note_text, user, apply_all=False, **kwargs):
+        """
+        Add a note to the given order item.
+
+        :param apply_all: If set, the note should be added to all
+           items on the order, instead of just the one item given.
+        """
+        model = self.model
+
+        if apply_all:
+            items = item.order.items
+        else:
+            items = [item]
+
+        occurred = kwargs.get('occurred')
+        for item in items:
+            item.events.append(model.CustomerOrderItemEvent(
+                type_code=self.enum.CUSTORDER_ITEM_EVENT_NOTE_ADDED,
+                user=user, note=note_text, occurred=occurred))
