@@ -3,14 +3,10 @@
 # This code was (mostly, with some tweaks) stolen from:
 # http://www.jejik.com/articles/2007/02/a_simple_unix_linux_daemon_in_python/
 
-from __future__ import absolute_import
-
 import sys, os, time, atexit
 import stat
 from signal import SIGTERM 
 
-import six
-
 
 class Daemon(object):
     """
@@ -69,7 +65,7 @@ class Daemon(object):
 
         # write pidfile
         atexit.register(self.delpid)
-        pid = six.text_type(os.getpid())
+        pid = str(os.getpid())
         with open(self.pidfile, 'w+') as f:
             f.write('{}\n'.format(pid))
         os.chmod(self.pidfile, stat.S_IRUSR|stat.S_IWUSR)
@@ -129,12 +125,12 @@ class Daemon(object):
                 os.kill(pid, SIGTERM)
                 time.sleep(0.1)
         except OSError as err:
-            err = six.text_type(err)
+            err = str(err)
             if err.find("No such process") > 0:
                 if os.path.exists(self.pidfile):
                     os.remove(self.pidfile)
             else:
-                print(six.text_type(err))
+                print(str(err))
                 sys.exit(1)
 
     def restart(self):
