@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,10 +24,9 @@
 DataSync Configuration
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import re
 import logging
+import warnings
 
 from rattail.config import ConfigProfile
 from rattail.exceptions import ConfigurationError
@@ -51,9 +50,19 @@ class DataSyncProfile(ConfigProfile):
 
     def __init__(self, *args, **kwargs):
         load_disabled_consumers = kwargs.pop('load_disabled_consumers', False)
-        super(DataSyncProfile, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
-        self.watcher_spec = self._config_string('watcher')
+        self.watcher_spec = self._config_string('watcher.spec')
+        if not self.watcher_spec:
+            # TODO: remove this fallback someday, since it won't work
+            # with newer config logic
+            self.watcher_spec = self._config_string('watcher', ignore_ambiguous=True)
+            if self.watcher_spec:
+                warnings.warn(f"URGENT: instead of '{self.section}.{self.prefix}.watcher', "
+                              f"you should set '{self.section}.{self.prefix}.watcher.spec'",
+                              DeprecationWarning, stacklevel=2)
+            else:
+                raise RuntimeError("no watcher spec defined")
 
         if self.watcher_spec == 'null':
             self.watcher = NullWatcher(self.config, self.key)
@@ -80,16 +89,12 @@ class DataSyncProfile(ConfigProfile):
             else: # config file
 
                 pattern = re.compile(r'^{}\.watcher\.kwarg\.(?P<keyword>\w+)$'.format(self.key))
-                # TODO: this should not be referencing the config parser directly?
-                # (but we have no other way yet, to know which options are defined)
-                # (we should at least allow config to be defined in DB Settings)
-                # (however that should be optional, since some may not have a DB)
-                if self.config.parser.has_section(self.section):
-                    for option in self.config.parser.options(self.section):
-                        match = pattern.match(option)
-                        if match:
-                            keyword = match.group('keyword')
-                            kwargs[keyword] = self.config.get(self.section, option)
+                settings = self.config.get_dict(self.section)
+                for key in settings:
+                    match = pattern.match(key)
+                    if match:
+                        keyword = match.group('keyword')
+                        kwargs[keyword] = settings[key]
 
             # only capture "explicitly configured" watcher kwargs..
             # (nb. this is used for configuration of the kwargs)
@@ -108,7 +113,14 @@ class DataSyncProfile(ConfigProfile):
         self.watcher.retry_delay = self._config_int('watcher.retry_delay', default=self.watcher.retry_delay)
         self.watcher.default_runas = self._config_string('consumers.runas')
 
-        consumers = self._config_list('consumers')
+        consumers = self._config_list('consumers.list')
+        if not consumers:
+            consumers = self._config_list('consumers', ignore_ambiguous=True)
+            if consumers:
+                warnings.warn(f"URGENT: instead of 'rattail.datasync.{self.key}.consumers', "
+                              f"you should set 'rattail.datasync.{self.key}.consumers.list'",
+                              DeprecationWarning, stacklevel=2)
+
         if consumers == ['self']:
             self.watcher.consumes_self = True
         else:
@@ -125,9 +137,20 @@ class DataSyncProfile(ConfigProfile):
                                         include_disabled=False)
         for key in get_consumer_keys(self.config, self.key,
                                      include_disabled=include_disabled):
-            consumer_spec = self._config_string('consumer.{}'.format(key))
+
+            consumer_spec = self._config_string(f'consumer.{key}.spec')
+            if not consumer_spec:
+                consumer_spec = self._config_string(f'consumer.{key}', ignore_ambiguous=True)
+                if consumer_spec:
+                    warnings.warn(f"URGENT: instead of '{self.section}.{self.prefix}.consumer.{key}', "
+                                  f"you should set '{self.section}.{self.prefix}.consumer.{key}.spec'",
+                                  DeprecationWarning, stacklevel=2)
+                else:
+                    raise RuntimeError(f"must define '{self.section}.{self.prefix}.consumer.{key}.spec'")
+
             if consumer_spec == 'null':
                 consumer_spec = 'rattail.datasync.consumers:NullTestConsumer'
+
             dbkey = self._config_string('consumer.{}.db'.format(key),
                                         default='default')
             runas = self._config_string('consumer.{}.runas'.format(key))
@@ -161,29 +184,38 @@ class DataSyncProfile(ConfigProfile):
 def get_consumer_keys(config, profile_key, include_disabled=False):
 
     # start with the primary set of consumer keys
-    keys = config.getlist('rattail.datasync', '{}.consumers'.format(profile_key))
+    keys = config.getlist('rattail.datasync', f'{profile_key}.consumers.list')
+    if not keys:
+        keys = config.getlist('rattail.datasync', f'{profile_key}.consumers',
+                              ignore_ambiguous=True)
+        if keys:
+            warnings.warn(f"URGENT: instead of 'rattail.datasync.{profile_key}.consumers', "
+                          f"you should set 'rattail.datasync.{profile_key}.consumers.list'",
+                          DeprecationWarning, stacklevel=2)
+        else:
+            keys = []
 
     if include_disabled:
 
         # first look in config file options
-        if config.parser.has_section('rattail.datasync'):
+        settings = config.get_dict('rattail.datasync')
+        pattern = re.compile(r'^{}\.consumer\.([^.]+)\.spec$'.format(profile_key))
+        for key in settings:
 
             # find all consumers with spec defined
-            pattern = re.compile(r'^{}\.consumer\.([^.]+)$'.format(profile_key))
-            for option in config.parser.options('rattail.datasync'):
-                match = pattern.match(option)
-                if match:
-                    keys.append(match.group(1))
+            match = pattern.match(key)
+            if match:
+                keys.append(match.group(1))
 
         # maybe also look for config settings in DB
         if config.usedb:
             app = config.get_app()
-            model = config.get_model()
+            model = app.model
             session = app.make_session()
             settings = session.query(model.Setting)\
-                              .filter(model.Setting.name.like('rattail.datasync.{}.consumer.%'.format(profile_key)))\
+                              .filter(model.Setting.name.like(f'rattail.datasync.{profile_key}.consumer.%.spec'))\
                               .all()
-            pattern = re.compile(r'^rattail\.datasync\.{}\.consumer\.([^.]+)$')
+            pattern = re.compile(r'^rattail\.datasync\.{}\.consumer\.(.+)\.spec$')
             for setting in settings:
                 match = pattern.match(setting.name)
                 if match:
@@ -204,26 +236,26 @@ def get_profile_keys(config, include_disabled=False):
     if include_disabled:
 
         # first look in config file options
-        if config.parser.has_section('rattail.datasync'):
+        settings = config.get_dict('rattail.datasync')
+        pattern = re.compile(r'^(\S+)\.watcher\.spec$')
+        for key in settings:
 
             # find all profiles with watcher defined
-            pattern = re.compile(r'^(\S+)\.watcher$')
-            for option in config.parser.options('rattail.datasync'):
-                match = pattern.match(option)
-                if match:
-                    keys.append(match.group(1))
+            match = pattern.match(key)
+            if match:
+                keys.append(match.group(1))
 
         # maybe also look for config settings in DB
         if config.usedb:
             app = config.get_app()
-            model = config.get_model()
+            model = app.model
             session = app.make_session()
             settings = session.query(model.Setting)\
-                              .filter(model.Setting.name.like('rattail.datasync.%.watcher'))\
+                              .filter(model.Setting.name.like('rattail.datasync.%.watcher.spec'))\
                               .all()
             for setting in settings:
                 parts = setting.name.split('.')
-                keys.append('.'.join(parts[2:-1]))
+                keys.append('.'.join(parts[2:-2]))
             session.close()
 
     return list(sorted(set(keys)))
@@ -256,7 +288,7 @@ def load_profiles(config, include_disabled=False, ignore_problems=False):
                 profile = DataSyncProfile(config, key,
                                           load_disabled_consumers=True)
             except Exception as error:
-                log.debug("could not create '%s' profile", key, exc_info=True)
+                log.warning("could not create '%s' profile", key, exc_info=True)
                 pass
             else:
                 profile.enabled = key in enabled
