@@ -41,7 +41,7 @@ class RattailWatcher(DataSyncWatcher):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.engine = self.config.rattail_engines[self.dbkey]
+        self.engine = self.config.appdb_engines[self.dbkey]
         self.topo_sortkey = make_topo_sortkey(self.app.model)
 
     def get_changes(self, lastrun):
@@ -176,7 +176,7 @@ class RattailConsumer(DataSyncConsumer):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.engine = self.config.rattail_engines[self.dbkey]
+        self.engine = self.config.appdb_engines[self.dbkey]
         self.model = self.get_data_model()
         self.topo_sortkey = make_topo_sortkey(self.model)
 
@@ -429,7 +429,7 @@ class FromRattailToRattailExportConsumer(FromRattailToRattailBase):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.target_engine = self.config.rattail_engines[self.dbkey]
+        self.target_engine = self.config.appdb_engines[self.dbkey]
         self.model = self.app.model
 
     def make_target_session(self):
@@ -471,8 +471,8 @@ class FromRattailToRattailImportConsumer(FromRattailToRattailBase):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.host_engine = self.config.rattail_engines[self.watcher.dbkey]
-        self.local_engine = self.config.rattail_engines[self.dbkey]
+        self.host_engine = self.config.appdb_engines[self.watcher.dbkey]
+        self.local_engine = self.config.appdb_engines[self.dbkey]
         self.model = self.app.model
 
     def process_changes(self, session, changes):
