@@ -24,24 +24,16 @@
 Products API
 """
 
-import logging
 import warnings
 
 from sqlalchemy import orm
-from sqlalchemy.orm.exc import NoResultFound
 
 from rattail import enum
 from rattail.db import model
 
 
-log = logging.getLogger(__name__)
-
-
 def get_product_by_upc(session, upc, include_deleted=False):
-    """
-    Returns the :class:`rattail.Product` associated with ``upc`` (if found), or
-    ``None``.
-    """
+    """ DEPRECATED """
     warnings.warn("get_product_by_upc() is deprecated; please use "
                   "ProductHandler.locate_product_for_gpc() instead",
                   DeprecationWarning, stacklevel=2)
@@ -51,53 +43,46 @@ def get_product_by_upc(session, upc, include_deleted=False):
         products = products.filter(model.Product.deleted == False)
     try:
         return products.filter(model.Product.upc == upc).one()
-    except NoResultFound:
+    except orm.exc.NoResultFound:
         pass
 
 
 def get_product_by_item_id(session, item_id, include_deleted=False):
-    """
-    Returns the :class:`rattail.Product` associated with ``item_id`` (if
-    found), or ``None``.
-    """
+    """ DEPRECATED """
+    warnings.warn("get_product_by_item_id() is deprecated; please use "
+                  "ProductHandler.locate_product_for_item_id() instead",
+                  DeprecationWarning, stacklevel=2)
+
     products = session.query(model.Product)
     if not include_deleted:
         products = products.filter(model.Product.deleted == False)
     try:
         return products.filter(model.Product.item_id == item_id).one()
-    except NoResultFound:
+    except orm.exc.NoResultFound:
         pass
 
 
 def get_product_by_scancode(session, scancode, include_deleted=False):
-    """
-    Returns the :class:`rattail.Product` associated with ``scancode`` (if
-    found), or ``None``.
-    """
+    """ DEPRECATED """
+    warnings.warn("get_product_by_scancode() is deprecated; please use "
+                  "ProductHandler.locate_product_for_scancode() instead",
+                  DeprecationWarning, stacklevel=2)
+
     products = session.query(model.Product)
     if not include_deleted:
         products = products.filter(model.Product.deleted == False)
     try:
         return products.filter(model.Product.scancode == scancode).one()
-    except NoResultFound:
+    except orm.exc.NoResultFound:
         pass
 
 
 def get_product_by_code(session, code, include_deleted=False):
-    """
-    Locate a ``Product`` instance using a code value.
+    """ DEPRECATED """
+    warnings.warn("get_product_by_code() is deprecated; please use "
+                  "ProductHandler.locate_product_for_alt_code() instead",
+                  DeprecationWarning, stacklevel=2)
 
-    :param session: An open SQLAlchemy session.
-
-    :param code: Product code for which to search.
-    :type code: string
-
-    :returns: A :class:`rattail.db.model.Product` instance, or ``None``.
-
-    .. note::
-       This function will return the *first* product found, and does not warn
-       if multiple products would have matched the search.
-    """
     products = session.query(model.Product).join(model.ProductCode)
     if not include_deleted:
         products = products.filter(model.Product.deleted == False)
@@ -105,23 +90,11 @@ def get_product_by_code(session, code, include_deleted=False):
 
 
 def get_product_by_vendor_code(session, code, vendor=None, include_deleted=False):
-    """
-    Locate a product using a vendor item code, optionally restricted to a
-    particular vendor.
+    """ DEPRECATED """
+    warnings.warn("get_product_by_vendor_code() is deprecated; please use "
+                  "ProductHandler.locate_product_for_vendor_code() instead",
+                  DeprecationWarning, stacklevel=2)
 
-    :param session: A SQLAlchemy session.
-
-    :param code: Vendor item code for which to search.
-    :type code: string
-
-    :param vendor: Optional vendor instance by which to filter the search.
-
-    :returns: A :class:`rattail.db.model.Product` instance, or ``None``.
-
-    .. note::
-       This function will return the *first* product found, and does not warn
-       if multiple products would have matched the search.
-    """
     products = session.query(model.Product).join(model.ProductCost)
     if vendor:
         products = products.filter(model.ProductCost.vendor == vendor)
@@ -196,29 +169,11 @@ future_cost_fieldmap = {
 
 
 def make_future_cost_current(config, future):
-    """
-    Take the given future cost, and make it current.
-    """
-    if not future.cost:
-        raise NotImplementedError("don't know how to create costs yet")
+    """ DEPRECATED """
+    warnings.warn("get_product_by_upc() is deprecated; please use "
+                  "ProductHandler.locate_product_for_gpc() instead",
+                  DeprecationWarning, stacklevel=2)
 
-    cost = future.cost
-
-    if not future.product:
-        raise ValueError("future cost does not have product association: {}".format(future))
-
-    if not future.vendor:
-        raise ValueError("future cost does not have vendor association: {}".format(future))
-
-    if future.product is not cost.product:
-        raise ValueError("product mismatch for future/current costs: {}".format(future))
-
-    if future.vendor is not cost.vendor:
-        raise ValueError("vendor mismatch for future/current costs: {}".format(future))
-
-    for field in future_cost_fieldmap:
-        setattr(cost, future_cost_fieldmap[field], getattr(future, field))
-
-    session = orm.object_session(future)
-    session.delete(future)
-    log.debug("future cost became current: %s", cost)
+    app = config.get_app()
+    products_handler = app.get_products_handler()
+    return products_handler.make_future_cost_current(future)
