@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -21,11 +21,15 @@
 #
 ################################################################################
 """
-Authentication & Authorization
+DEPRECATED: Authentication & Authorization
+
+This entire module has been deprecated; please use the :term:`auth
+handler` instead.
 """
 
+import warnings
+
 from passlib.context import CryptContext
-from sqlalchemy.orm.exc import NoResultFound
 
 from rattail.db import model
 
@@ -35,23 +39,21 @@ password_context = CryptContext(schemes=['bcrypt'])
 
 def authenticate_user(session, userobj, password):
     """
-    Attempt to authenticate a user.
-
-    :param userobj: May be a :class:`model.User` instance, or a username as
-       string.  If the latter, it will be used to look up the User instance.
-
-    :returns: The User instance, if found and the password was correct;
-       otherwise ``None``.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.authenticate_user()`
+    instead.
     """
+    warnings.warn("authenticate_user() function is deprecated; "
+                  "please use AuthHandler.authenticate_user() instead",
+                  DeprecationWarning, stacklevel=2)
+
     if isinstance(userobj, model.User):
         user = userobj
     else:
-        try:
-            user = session.query(model.User)\
-                          .filter_by(username=userobj)\
-                          .one()
-        except NoResultFound:
-            user = None
+        user = session.query(model.User)\
+                      .filter_by(username=userobj)\
+                      .first()
+
     if user and user.active and user.password is not None:
         if password_context.verify(password, user.password):
             return user
@@ -59,15 +61,22 @@ def authenticate_user(session, userobj, password):
 
 def set_user_password(user, password):
     """
-    Set a user's password.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.set_user_password()`
+    instead.
     """
+    warnings.warn("set_user_password() function is deprecated; "
+                  "please use AuthHandler.set_user_password() instead",
+                  DeprecationWarning, stacklevel=2)
+
     user.password = password_context.hash(password)
 
 
 def special_role(session, uuid, name):
-    """
-    Fetches, or creates, a "special" role.
-    """
+    """ """
+    warnings.warn("special_role() function is deprecated; "
+                  "please use AuthHandler._special_role() instead",
+                  DeprecationWarning, stacklevel=2)
 
     role = session.get(model.Role, uuid)
     if not role:
@@ -78,31 +87,52 @@ def special_role(session, uuid, name):
 
 def administrator_role(session):
     """
-    Returns the "Administrator" role.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.get_role_administrator()`
+    instead.
     """
+    warnings.warn("administrator_role() function is deprecated; "
+                  "please use AuthHandler.get_role_administrator() instead",
+                  DeprecationWarning, stacklevel=2)
 
     return special_role(session, 'd937fa8a965611dfa0dd001143047286', 'Administrator')
 
 
 def guest_role(session):
     """
-    Returns the "Guest" role.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.get_role_anonymous()`
+    instead.
     """
+    warnings.warn("guest_role() function is deprecated; "
+                  "please use AuthHandler.get_role_anonymous() instead",
+                  DeprecationWarning, stacklevel=2)
 
     return special_role(session, 'f8a27c98965a11dfaff7001143047286', 'Guest')
 
 
 def authenticated_role(session):
     """
-    Returns the "Authenticated" role.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.get_role_authenticated()`
+    instead.
     """
+    warnings.warn("authenticated_role() function is deprecated; "
+                  "please use AuthHandler.get_role_authenticated() instead",
+                  DeprecationWarning, stacklevel=2)
+
     return special_role(session, 'b765a9cc331a11e6ac2a3ca9f40bc550', "Authenticated")
 
 
 def grant_permission(role, permission):
     """
-    Grant a permission to a role.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.grant_permission()`
+    instead.
     """
+    warnings.warn("grant_permission() function is deprecated; "
+                  "please use AuthHandler.grant_permission() instead",
+                  DeprecationWarning, stacklevel=2)
 
     # TODO: Make this a `Role` method (or make `Role.permissions` a `set` so we
     # can do `role.permissions.add('some.perm')` ?).
@@ -112,38 +142,27 @@ def grant_permission(role, permission):
 
 def revoke_permission(role, permission):
     """
-    Revoke the given permission for the given role.  This first checks to see
-    if the role currently has the permission; if not then no change is made.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.revoke_permission()`
+    instead.
     """
+    warnings.warn("revoke_permission() function is deprecated; "
+                  "please use AuthHandler.revoke_permission() instead",
+                  DeprecationWarning, stacklevel=2)
+
     if permission in role.permissions:
         role.permissions.remove(permission)
 
 
 def has_permission(session, principal, permission, include_guest=True, include_authenticated=True):
     """
-    Determine if a principal has been granted a permission.
-
-    :param session: A SQLAlchemy session instance.
-
-    :param principal: May be either a :class:`.model.User` or
-       :class:`.model.Role` instance.  It is also expected that this may
-       sometimes be ``None``, in which case the "Guest" role will typically be
-       assumed.
-
-    :param permission: The full internal name of a permission,
-       e.g. ``'users.create'``.
-
-    :param include_guest: Whether or not the "Guest" role should be included
-       when checking permissions.  If ``False``, then Guest's permissions will
-       *not* be consulted.
-
-    :param include_authenticated: Whether or not the "Authenticated" role
-       should be included when checking permissions.
-
-    Note that if no ``principal`` is provided, and ``include_guest`` is set to
-    ``False``, then no checks will actually be done, and the return value will
-    be ``False``.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.has_permission()`
+    instead.
     """
+    warnings.warn("has_permission() function is deprecated; "
+                  "please use AuthHandler.has_permission() instead",
+                  DeprecationWarning, stacklevel=2)
 
     if hasattr(principal, 'roles'):
         roles = list(principal.roles)
@@ -165,27 +184,14 @@ def has_permission(session, principal, permission, include_guest=True, include_a
 
 def cache_permissions(session, principal, include_guest=True, include_authenticated=True):
     """
-    Return a set of permission names, which represents all permissions
-    effectively granted to the given principal.
-
-    :param session: A SQLAlchemy session instance.
-
-    :param principal: May be either a :class:`.model.User` or
-       :class:`.model.Role` instance.  It is also expected that this may
-       sometimes be ``None``, in which case the "Guest" role will typically be
-       assumed.
-
-    :param include_guest: Whether or not the "Guest" role should be included
-       when checking permissions.  If ``False``, then Guest's permissions will
-       *not* be consulted.
-
-    :param include_authenticated: Whether or not the "Authenticated" role
-       should be included when checking permissions.
-
-    Note that if no ``principal`` is provided, and ``include_guest`` is set to
-    ``False``, then no checks will actually be done, and the return value will
-    be ``False``.
+    DEPRECATED; use
+    :meth:`wuttjamaican:wuttjamaican.auth.AuthHandler.get_permissions()`
+    instead.
     """
+    warnings.warn("cache_permissions() function is deprecated; "
+                  "please use AuthHandler.get_permissions() instead",
+                  DeprecationWarning, stacklevel=2)
+
     # we will use any `roles` attribute which may be present.  in practice we
     # would be assuming a User in this case
     if hasattr(principal, 'roles'):
