@@ -30,17 +30,13 @@ from packaging.version import parse as parse_version
 import sqlalchemy as sa
 from sqlalchemy.orm import object_mapper, RelationshipProperty
 from sqlalchemy.orm.session import Session
-
-try:
-    from sqlalchemy.event import listen
-except ImportError:
-    listen = None # old SQLAlchemy; will have to work around that, below
+from sqlalchemy.event import listen
 
 from rattail.db.model import Setting, Change, DataSyncChange
 
 try:
     from rattail.db.continuum import versioning_manager
-except ImportError:             # assume no continuum
+except ImportError: # pragma: no cover
     versioning_manager = None
 
 
@@ -49,10 +45,14 @@ log = logging.getLogger(__name__)
 
 def record_changes(session, recorder=None, config=None):
     """
-    Record all relevant data changes which occur within a session.
+    Turn on the "record changes" feature.
 
-    :param session: A :class:`sqlalchemy:sqlalchemy.orm.session.Session` class,
-       or instance thereof.
+    With this enabled, all relevant data changes which occur in the
+    sesion will be recorded.
+
+    :param session: A
+       :class:`sqlalchemy:sqlalchemy.orm.session.Session` class, or
+       instance thereof.
     """
     if isinstance(recorder, ChangeRecorder):
         pass
@@ -61,23 +61,15 @@ def record_changes(session, recorder=None, config=None):
     elif recorder is None:
         if config:
             app = config.get_app()
-            spec = config.get('rattail.db', 'changes.recorder', usedb=False)
+            spec = config.get('rattail.db.changes.recorder', usedb=False)
             if spec:
                 recorder = app.load_object(spec)(config)
         if not recorder:
             recorder = ChangeRecorder(config)
     else:
-        raise ValueError("Invalid 'recorder' parameter: {}".format(repr(recorder)))
-
-    if listen:
-        listen(session, 'before_flush', recorder)
-    else:
-        extension = ChangeRecorderExtension(recorder)
-        if isinstance(session, Session):
-            session.extensions.append(extension)
-        else:
-            session.configure(extension=extension)
+        raise ValueError(f"recorder not valid: {recorder}")
 
+    listen(session, 'before_flush', recorder)
     session.rattail_record_changes = True
     session.rattail_change_recorder = recorder
 
