@@ -28,6 +28,7 @@ import logging
 import warnings
 
 from wuttjamaican.util import parse_bool
+from wuttjamaican.db import get_engines as wutta_get_engines
 from wuttjamaican.db.conf import make_engine_from_config as wutta_make_engine_from_config
 
 from rattail.exceptions import SQLAlchemyNotInstalled
@@ -44,8 +45,7 @@ def get_engines(config, section='rattail.db'):
     warnings.warn("rattail.db.config.get_engines() is deprecated; "
                   "please use wuttjamaican.db.get_engines() instead",
                   DeprecationWarning, stacklevel=2)
-    from wuttjamaican.db import get_engines
-    return get_engines(config, section)
+    return wutta_get_engines(config, section)
 
 
 def get_default_engine(config, section='rattail.db'):
@@ -56,17 +56,17 @@ def get_default_engine(config, section='rattail.db'):
     warnings.warn("rattail.db.config.get_default_engine() is deprecated; "
                   "please use wuttjamaican.db.get_engines() instead",
                   DeprecationWarning, stacklevel=2)
-    from wuttjamaican.db import get_engines
-    return get_engines(config, section).get('default')
+    return wutta_get_engines(config, section).get('default')
 
 
-# TODO: Deprecate/remove this.
+# TODO: DEPRECATED - this should be removed soon
 def configure_session(config, session):
-    """
-    Configure a session factory or instance.  Currently all this does is
-    install the hook to record changes, if config so dictates.
-    """
+    """ """
     if config.getbool('rattail.db', 'changes.record', usedb=False):
+        warnings.warn("setting rattail.db.changes.record is deprecated; "
+                      "please set per-engine .record_changes instead",
+                      DeprecationWarning)
+
         from rattail.db.changes import record_changes
         record_changes(session, config=config)
 
@@ -75,7 +75,7 @@ def configure_versioning(config, force=False, manager=None, plugins=None, **kwar
     """
     Configure Continuum versioning.
     """
-    if not config.versioning_enabled() and not force:
+    if not (force or config.versioning_enabled()):
         return
 
     try:
@@ -97,9 +97,19 @@ def configure_versioning(config, force=False, manager=None, plugins=None, **kwar
         # TODO: is this the best way/place to confirm versioning?
         app = config.get_app()
         try:
+            # TODO: not sure why but if we don't load app.model here,
+            # for some reason the alembic upgrade command may fail, if
+            # any app models include the 'active_history' feature?!
             model = app.model
+
+            # but when confirming if versioning is working, we
+            # definitely need to check a "native" model and not
+            # anything from app.model, which may use a totally
+            # different base class with no versioning (i.e. wutta)
+            from rattail.db.model import User
+
             configure_mappers()
-            transaction_class = continuum.transaction_class(model.User)
+            transaction_class = continuum.transaction_class(User)
             config.versioning_has_been_enabled = True
         except continuum.ClassNotVersioned:
             raise RuntimeError("Versioning is enabled and configured, but is not functional!  "
