@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -22,15 +22,19 @@
 ################################################################################
 """
 Rattail data models
+
+This namespace will be populated with all data model classes defined
+by Rattail.
 """
 
-from .core import Base, ModelBase, uuid_column, getset_factory, GPCType, Setting, Change, Note
+from .core import (uuid_column, uuid_fk_column,
+                   Base, ModelBase, getset_factory, GPCType, Setting, Change, Note)
 from .contact import PhoneNumber, EmailAddress, MailingAddress
 
 from .people import (Person, PersonNote,
                      PersonPhoneNumber, PersonEmailAddress, PersonMailingAddress,
                      MergePeopleRequest)
-from .users import Role, Permission, User, UserRole, UserEvent
+from .users import Role, Permission, User, UserRole, UserEvent, UserAPIToken
 from .stores import Store, StorePhoneNumber, StoreEmailAddress
 from .customers import (Customer,
                         CustomerPhoneNumber,
@@ -38,10 +42,17 @@ from .customers import (Customer,
                         CustomerMailingAddress,
                         CustomerGroup,
                         CustomerGroupAssignment,
+                        CustomerShopper,
+                        CustomerShopperHistory,
                         CustomerPerson,
                         CustomerNote,
                         PendingCustomer)
-from .members import Member, MemberPhoneNumber, MemberEmailAddress, MemberMailingAddress
+from .members import (MembershipType,
+                      Member,
+                      MemberPhoneNumber,
+                      MemberEmailAddress,
+                      MemberMailingAddress,
+                      MemberEquityPayment)
 
 from .org import Department, Subdepartment, Category, Family, ReportCode, DepositLink
 from .employees import (Employee, EmployeePhoneNumber, EmployeeEmailAddress,
@@ -57,6 +68,7 @@ from .ifps import IFPS_PLU
 from .purchase import (PurchaseBase, PurchaseItemBase, PurchaseCreditBase,
                        Purchase, PurchaseItem, PurchaseCredit)
 
+from .sales import Tender
 from .custorders import (CustomerOrderBase, CustomerOrderItemBase,
                          CustomerOrder, CustomerOrderItem,
                          CustomerOrderItemEvent, CustomerOrderItemNote)
@@ -81,6 +93,7 @@ from .batch.inventory import InventoryBatch, InventoryBatchRow
 from .batch.labels import LabelBatch, LabelBatchRow
 from .batch.newproduct import NewProductBatch, NewProductBatchRow
 from .batch.delproduct import DeleteProductBatch, DeleteProductBatchRow
+from .batch.pos import POSBatch, POSBatchTax, POSBatchRow
 from .batch.pricing import PricingBatch, PricingBatchRow
 from .batch.product import ProductBatch, ProductBatchRow
 from .batch.purchase import PurchaseBatch, PurchaseBatchRow, PurchaseBatchRowClaim, PurchaseBatchCredit
