@@ -34,9 +34,9 @@ except ImportError:
     from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy.ext.associationproxy import association_proxy
 
-from wuttjamaican.db.model.base import naming_convention, uuid_column, uuid_fk_column
+from wuttjamaican.db.util import (naming_convention, uuid_column, uuid_fk_column,
+                                  ModelBase as WuttaModelBase)
 
-from rattail.core import Object
 from rattail.time import make_utc
 
 # These are imported because most sibling modules import them *from* here (if
@@ -46,7 +46,7 @@ from rattail.db.core import getset_factory
 from rattail.db.types import GPCType
 
 
-class ModelBase(Object):
+class ModelBase(WuttaModelBase):
     """
     Base class for all data models.
     """
