@@ -95,7 +95,11 @@ class Person(ContactMixin, Base):
     modified = sa.Column(sa.DateTime(), nullable=True, onupdate=datetime.datetime.utcnow)
 
     def __str__(self):
-        return self.display_name or ""
+        if self.display_name:
+            return self.display_name
+        if self.first_name or self.last_name:
+            return f"{self.first_name} {self.last_name}".strip()
+        return "(NO NAME!)"
 
     @property
     def user(self):
