@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2019 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,9 +24,6 @@
 Data models for reporting
 """
 
-from __future__ import unicode_literals, absolute_import
-
-import six
 import sqlalchemy as sa
 
 from rattail.db.model import Base, ExportMixin
@@ -35,7 +32,6 @@ from rattail.db.types import JSONTextDict
 from rattail.util import NOTSET
 
 
-@six.python_2_unicode_compatible
 class ReportOutput(ExportMixin, Base):
     """
     Represents a generated report(s)
@@ -60,7 +56,7 @@ class ReportOutput(ExportMixin, Base):
     def filepath(self, config, filename=NOTSET, **kwargs):
         if filename is NOTSET:
             filename = self.filename
-        return super(ReportOutput, self).filepath(config, filename=filename, **kwargs)
+        return super().filepath(config, filename=filename, **kwargs)
 
     def __str__(self):
         return "{} {}".format(self.id_str, self.report_name)
