@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2020 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,9 +24,6 @@
 Base data model for "shopfoo" pattern
 """
 
-from __future__ import unicode_literals, absolute_import
-
-import six
 import sqlalchemy as sa
 from sqlalchemy import orm
 from sqlalchemy.ext.declarative import declared_attr
@@ -68,12 +65,12 @@ class ShopfooProductBase(object):
             backref=orm.backref(
                 backref_name,
                 uselist=False,
+                cascade_backrefs=False,
                 doc="""
                 Reference to the local/cached {} record for this product.
                 """.format(backref_name)))
 
 
-@six.python_2_unicode_compatible
 class ShopfooProductExportBase(model.ExportMixin):
     """
     Product export table, specific to the target system.  Each record in this
