@@ -469,18 +469,17 @@ class upgrade_failure(Email):
     default_subject = "Upgrade failure for ${system_title}"
 
     def sample_data(self, request):
-        app = request.rattail_config.get_app()
-        upgrade = app.make_object(
+        upgrade = self.app.make_object(
             description="upgrade to the latest!",
             notes="nothing special",
-            executed=app.make_utc(),
+            executed=self.app.make_utc(),
             executed_by="Fred Flintstone",
             exit_code=42,
         )
         return {
             'upgrade': upgrade,
             'upgrade_url': '#',
-            'system_title': self.config.app_title(),
+            'system_title': self.app.get_title(),
         }
 
 
@@ -491,18 +490,17 @@ class upgrade_success(Email):
     default_subject = "Upgrade success for ${system_title}"
 
     def sample_data(self, request):
-        app = request.rattail_config.get_app()
-        upgrade = app.make_object(
+        upgrade = self.app.make_object(
             description="upgrade to the latest!",
             notes="nothing special",
-            executed=app.make_utc(),
+            executed=self.app.make_utc(),
             executed_by="Fred Flintstone",
             exit_code=0,
         )
         return {
             'upgrade': upgrade,
             'upgrade_url': '#',
-            'system_title': self.config.app_title(),
+            'system_title': self.app.get_title(),
         }
 
 
@@ -514,7 +512,7 @@ class user_feedback(Email):
 
     def sample_data(self, request):
         return {
-            'app_title': request.rattail_config.app_title(default="Rattail"),
+            'app_title': self.app.get_title(),
             'user': None,
             'user_name': "Fred Flintstone",
             'referrer': request.route_url('home'),
