@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -54,10 +54,12 @@ The following enumerations are provided:
    from the SIL specification.
 """
 
-from __future__ import unicode_literals, absolute_import
+from collections import OrderedDict
 
-from rattail.util import OrderedDict
 
+########################################
+# Batch Action
+########################################
 
 BATCH_ACTION_ADD                = 'ADD'
 BATCH_ACTION_ADD_REPLACE        = 'ADDRPL'
@@ -74,6 +76,10 @@ BATCH_ACTION = {
     }
 
 
+########################################
+# Customer Order Batch Mode
+########################################
+
 CUSTORDER_BATCH_MODE_CREATING            = 10
 CUSTORDER_BATCH_MODE_GATHERING           = 20
 
@@ -83,26 +89,40 @@ CUSTORDER_BATCH_MODE = {
 }
 
 
-CUSTORDER_STATUS_ORDERED                = 10
+########################################
+# Customer Order Status
+########################################
+
+CUSTORDER_STATUS_UNINITIATED            = 1
+CUSTORDER_STATUS_INITIATED              = 10
+# TODO: are these useful? not really tracking order status over time yet..
+# CUSTORDER_STATUS_ORDERED                = CUSTORDER_STATUS_INITIATED
 # CUSTORDER_STATUS_PAID                   = 20
 
-CUSTORDER_STATUS = {
-    CUSTORDER_STATUS_ORDERED            : "ordered",
+CUSTORDER_STATUS = OrderedDict([
+    (CUSTORDER_STATUS_UNINITIATED,      "uninitiated"),
+    (CUSTORDER_STATUS_INITIATED,        "initiated"),
+    # CUSTORDER_STATUS_ORDERED            : "ordered",
     # CUSTORDER_STATUS_PAID               : "paid",
-}
+])
 
 
+########################################
+# Customer Order Item Status
+########################################
+
+CUSTORDER_ITEM_STATUS_UNINITIATED       = 1
 CUSTORDER_ITEM_STATUS_INITIATED         = 10
+CUSTORDER_ITEM_STATUS_PAID_BEFORE       = 50
 # TODO: deprecate / remove this one
-CUSTORDER_ITEM_STATUS_ORDERED           = CUSTORDER_ITEM_STATUS_INITIATED
-CUSTORDER_ITEM_STATUS_READY             = 20
-# TODO: deprecate / remove this one
-CUSTORDER_ITEM_STATUS_PAID              = CUSTORDER_ITEM_STATUS_READY
-CUSTORDER_ITEM_STATUS_PLACED            = 30
-CUSTORDER_ITEM_STATUS_RECEIVED          = 40
-CUSTORDER_ITEM_STATUS_CONTACTED         = 50
-CUSTORDER_ITEM_STATUS_CONTACT_FAILED    = 60
-CUSTORDER_ITEM_STATUS_DELIVERED         = 70
+CUSTORDER_ITEM_STATUS_PAID              = CUSTORDER_ITEM_STATUS_PAID_BEFORE
+CUSTORDER_ITEM_STATUS_READY             = 100
+CUSTORDER_ITEM_STATUS_PLACED            = 200
+CUSTORDER_ITEM_STATUS_RECEIVED          = 300
+CUSTORDER_ITEM_STATUS_CONTACTED         = 350
+CUSTORDER_ITEM_STATUS_CONTACT_FAILED    = 375
+CUSTORDER_ITEM_STATUS_DELIVERED         = 500
+CUSTORDER_ITEM_STATUS_PAID_AFTER        = 550
 CUSTORDER_ITEM_STATUS_CANCELED          = 900
 CUSTORDER_ITEM_STATUS_REFUND_PENDING    = 910
 CUSTORDER_ITEM_STATUS_REFUNDED          = 920
@@ -111,14 +131,16 @@ CUSTORDER_ITEM_STATUS_EXPIRED           = 940
 CUSTORDER_ITEM_STATUS_INACTIVE          = 950
 
 CUSTORDER_ITEM_STATUS = OrderedDict([
-    (CUSTORDER_ITEM_STATUS_INITIATED,           "customer order initiated"),
-    # (CUSTORDER_ITEM_STATUS_PAID,                "payment received"),
-    (CUSTORDER_ITEM_STATUS_READY,               "ready to proceed"),
-    (CUSTORDER_ITEM_STATUS_PLACED,              "order placed with vendor"),
-    (CUSTORDER_ITEM_STATUS_RECEIVED,            "received from vendor"),
-    (CUSTORDER_ITEM_STATUS_CONTACTED,           "customer contacted"),
-    (CUSTORDER_ITEM_STATUS_CONTACT_FAILED,      "unable to contact customer"),
-    (CUSTORDER_ITEM_STATUS_DELIVERED,           "delivered to customer"),
+    (CUSTORDER_ITEM_STATUS_UNINITIATED,         "uninitiated"),
+    (CUSTORDER_ITEM_STATUS_INITIATED,           "initiated"),
+    (CUSTORDER_ITEM_STATUS_PAID_BEFORE,         "paid"),
+    (CUSTORDER_ITEM_STATUS_READY,               "ready"),
+    (CUSTORDER_ITEM_STATUS_PLACED,              "placed"),
+    (CUSTORDER_ITEM_STATUS_RECEIVED,            "received"),
+    (CUSTORDER_ITEM_STATUS_CONTACTED,           "contacted"),
+    (CUSTORDER_ITEM_STATUS_CONTACT_FAILED,      "contact failed"),
+    (CUSTORDER_ITEM_STATUS_DELIVERED,           "delivered"),
+    (CUSTORDER_ITEM_STATUS_PAID_AFTER,          "paid"),
     (CUSTORDER_ITEM_STATUS_CANCELED,            "canceled"),
     (CUSTORDER_ITEM_STATUS_REFUND_PENDING,      "refund pending"),
     (CUSTORDER_ITEM_STATUS_REFUNDED,            "refunded"),
@@ -128,50 +150,78 @@ CUSTORDER_ITEM_STATUS = OrderedDict([
 ])
 
 
+########################################
+# Customer Order Item Event Type
+########################################
+
 CUSTORDER_ITEM_EVENT_INITIATED          = 10
-CUSTORDER_ITEM_EVENT_PRICE_CONFIRMED    = 15
-CUSTORDER_ITEM_EVENT_READY              = 20
-# TODO: deprecate / remove this one
-CUSTORDER_ITEM_EVENT_PAID               = CUSTORDER_ITEM_EVENT_READY
-CUSTORDER_ITEM_EVENT_CUSTOMER_RESOLVED  = 21
-CUSTORDER_ITEM_EVENT_PRODUCT_RESOLVED   = 22
-CUSTORDER_ITEM_EVENT_PLACED             = 30
-CUSTORDER_ITEM_EVENT_RECEIVED           = 40
-CUSTORDER_ITEM_EVENT_CONTACTED          = 50
-CUSTORDER_ITEM_EVENT_CONTACT_FAILED     = 60
-CUSTORDER_ITEM_EVENT_DELIVERED          = 70
-CUSTORDER_ITEM_EVENT_STATUS_CHANGE      = 500 # nb. this is not in STATUS enum
-# CUSTORDER_ITEM_EVENT_ADDED_NOTE         = 510 # nb. this is not in STATUS enum
+CUSTORDER_ITEM_EVENT_PRICE_CONFIRMED    = 20
+CUSTORDER_ITEM_EVENT_PAYMENT_RECEIVED   = 50
+# TODO: deprecate / remove this
+CUSTORDER_ITEM_EVENT_PAID               = CUSTORDER_ITEM_EVENT_PAYMENT_RECEIVED
+CUSTORDER_ITEM_EVENT_READY              = 100
+CUSTORDER_ITEM_EVENT_CUSTOMER_RESOLVED  = 120
+CUSTORDER_ITEM_EVENT_PRODUCT_RESOLVED   = 140
+CUSTORDER_ITEM_EVENT_PLACED             = 200
+CUSTORDER_ITEM_EVENT_RECEIVED           = 300
+CUSTORDER_ITEM_EVENT_CONTACTED          = 350
+CUSTORDER_ITEM_EVENT_CONTACT_FAILED     = 375
+CUSTORDER_ITEM_EVENT_DELIVERED          = 500
+CUSTORDER_ITEM_EVENT_STATUS_CHANGE      = 700
+CUSTORDER_ITEM_EVENT_NOTE_ADDED         = 750
 CUSTORDER_ITEM_EVENT_CANCELED           = 900
 CUSTORDER_ITEM_EVENT_REFUND_PENDING     = 910
 CUSTORDER_ITEM_EVENT_REFUNDED           = 920
 CUSTORDER_ITEM_EVENT_RESTOCKED          = 930
 CUSTORDER_ITEM_EVENT_EXPIRED            = 940
 CUSTORDER_ITEM_EVENT_INACTIVE           = 950
+CUSTORDER_ITEM_EVENT_OTHER              = 999
 
 CUSTORDER_ITEM_EVENT = OrderedDict([
-    (CUSTORDER_ITEM_EVENT_INITIATED,            "customer order initiated"),
+    (CUSTORDER_ITEM_EVENT_INITIATED,            "initiated"),
     (CUSTORDER_ITEM_EVENT_PRICE_CONFIRMED,      "price confirmed"),
-    (CUSTORDER_ITEM_EVENT_READY,                "order becomes ready to proceed"),
-    # (CUSTORDER_ITEM_EVENT_PAID,                 "payment received"),
+    (CUSTORDER_ITEM_EVENT_PAYMENT_RECEIVED,     "payment received"),
+    (CUSTORDER_ITEM_EVENT_READY,                "ready to proceed"),
     (CUSTORDER_ITEM_EVENT_CUSTOMER_RESOLVED,    "customer resolved"),
     (CUSTORDER_ITEM_EVENT_PRODUCT_RESOLVED,     "product resolved"),
-    (CUSTORDER_ITEM_EVENT_PLACED,               "order placed with vendor"),
+    (CUSTORDER_ITEM_EVENT_PLACED,               "placed with vendor"),
     (CUSTORDER_ITEM_EVENT_RECEIVED,             "received from vendor"),
     (CUSTORDER_ITEM_EVENT_CONTACTED,            "customer contacted"),
-    (CUSTORDER_ITEM_EVENT_CONTACT_FAILED,       "unable to contact customer"),
-    (CUSTORDER_ITEM_EVENT_DELIVERED,            "delivered to customer"),
-    (CUSTORDER_ITEM_EVENT_STATUS_CHANGE,        "manual status change"),
-    # (CUSTORDER_ITEM_EVENT_ADDED_NOTE,           "added note"),
+    (CUSTORDER_ITEM_EVENT_CONTACT_FAILED,       "contact failed"),
+    (CUSTORDER_ITEM_EVENT_DELIVERED,            "delivered"),
+    (CUSTORDER_ITEM_EVENT_STATUS_CHANGE,        "status change"),
+    (CUSTORDER_ITEM_EVENT_NOTE_ADDED,           "note added"),
     (CUSTORDER_ITEM_EVENT_CANCELED,             "canceled"),
     (CUSTORDER_ITEM_EVENT_REFUND_PENDING,       "refund pending"),
     (CUSTORDER_ITEM_EVENT_REFUNDED,             "refunded"),
     (CUSTORDER_ITEM_EVENT_RESTOCKED,            "restocked"),
     (CUSTORDER_ITEM_EVENT_EXPIRED,              "expired"),
     (CUSTORDER_ITEM_EVENT_INACTIVE,             "inactive"),
+    (CUSTORDER_ITEM_EVENT_OTHER,                "other"),
 ])
 
 
+########################################
+# Person Note Type
+########################################
+
+PERSON_NOTE_TYPE_PERSONAL       = 'personal'
+PERSON_NOTE_TYPE_MEMBER         = 'member'
+PERSON_NOTE_TYPE_CUSTOMER       = 'customer'
+PERSON_NOTE_TYPE_EMPLOYEE       = 'employee'
+
+PERSON_NOTE_TYPE = OrderedDict([
+    (PERSON_NOTE_TYPE_PERSONAL,         "Personal"),
+    (PERSON_NOTE_TYPE_MEMBER,           "Member"),
+    (PERSON_NOTE_TYPE_CUSTOMER,         "Customer"),
+    (PERSON_NOTE_TYPE_EMPLOYEE,         "Employee"),
+])
+
+
+########################################
+# Work Order Status
+########################################
+
 # WORKORDER_STATUS_CREATED                 = 1
 WORKORDER_STATUS_SUBMITTED               = 10
 WORKORDER_STATUS_RECEIVED                = 20
@@ -195,6 +245,10 @@ WORKORDER_STATUS = OrderedDict([
 ])
 
 
+########################################
+# Work Order Event
+########################################
+
 # WORKORDER_EVENT_CREATED                 = 1
 WORKORDER_EVENT_SUBMITTED               = 10
 WORKORDER_EVENT_RECEIVED                = 20
@@ -218,6 +272,10 @@ WORKORDER_EVENT = OrderedDict([
 ])
 
 
+########################################
+# Email Attempt Status
+########################################
+
 EMAIL_ATTEMPT_CREATED           = 0
 EMAIL_ATTEMPT_SUCCESS           = 1
 EMAIL_ATTEMPT_FAILURE           = 2
@@ -231,6 +289,10 @@ EMAIL_ATTEMPT = {
 }
 
 
+########################################
+# Email Preference
+########################################
+
 EMAIL_PREFERENCE_NONE           = 0
 EMAIL_PREFERENCE_TEXT           = 1
 EMAIL_PREFERENCE_HTML           = 2
@@ -244,6 +306,10 @@ EMAIL_PREFERENCE = {
     }
 
 
+########################################
+# Handheld Device Type
+########################################
+
 HANDHELD_DEVICE_TYPE_MOTOROLA           = 'motorola'
 HANDHELD_DEVICE_TYPE_PALMOS             = 'palmos'
 
@@ -253,6 +319,10 @@ HANDHELD_DEVICE_TYPE = {
 }
 
 
+########################################
+# Importer Batch Row Status
+########################################
+
 IMPORTER_BATCH_ROW_STATUS_NOCHANGE      = 0
 IMPORTER_BATCH_ROW_STATUS_CREATE        = 1
 IMPORTER_BATCH_ROW_STATUS_UPDATE        = 2
@@ -266,6 +336,10 @@ IMPORTER_BATCH_ROW_STATUS = {
 }
 
 
+########################################
+# Inventory Mode
+########################################
+
 INVENTORY_MODE_REPLACE          = 1
 INVENTORY_MODE_REPLACE_ADJUST   = 2
 INVENTORY_MODE_ADJUST           = 3
@@ -281,6 +355,10 @@ INVENTORY_MODE = {
 }
 
 
+########################################
+# Message Status
+########################################
+
 MESSAGE_STATUS_INBOX            = 1
 MESSAGE_STATUS_ARCHIVE          = 2
 
@@ -290,6 +368,10 @@ MESSAGE_STATUS = {
 }
 
 
+########################################
+# Pending Customer Status
+########################################
+
 PENDING_CUSTOMER_STATUS_PENDING         = 1
 PENDING_CUSTOMER_STATUS_READY           = 2
 PENDING_CUSTOMER_STATUS_RESOLVED        = 3
@@ -301,17 +383,27 @@ PENDING_CUSTOMER_STATUS = OrderedDict([
 ])
 
 
+########################################
+# Pending Product Status
+########################################
+
 PENDING_PRODUCT_STATUS_PENDING          = 1
 PENDING_PRODUCT_STATUS_READY            = 2
 PENDING_PRODUCT_STATUS_RESOLVED         = 3
+PENDING_PRODUCT_STATUS_IGNORED          = 4
 
 PENDING_PRODUCT_STATUS = OrderedDict([
     (PENDING_PRODUCT_STATUS_PENDING,    "pending"),
     (PENDING_PRODUCT_STATUS_READY,      "ready"),
     (PENDING_PRODUCT_STATUS_RESOLVED,   "resolved"),
+    (PENDING_PRODUCT_STATUS_IGNORED,    "ignored"),
 ])
 
 
+########################################
+# Phone Type
+########################################
+
 PHONE_TYPE_HOME                 = 'home'
 PHONE_TYPE_MOBILE               = 'mobile'
 PHONE_TYPE_OTHER                = 'other'
@@ -323,6 +415,47 @@ PHONE_TYPE = {
     }
 
 
+########################################
+# POS Batch Row Type
+########################################
+
+POS_ROW_TYPE_SET_CUSTOMER       = 'set_customer'
+POS_ROW_TYPE_SWAP_CUSTOMER      = 'swap_customer'
+POS_ROW_TYPE_DEL_CUSTOMER       = 'del_customer'
+POS_ROW_TYPE_SELL               = 'sell'
+POS_ROW_TYPE_OPEN_RING          = 'openring'
+POS_ROW_TYPE_BADSCAN            = 'badscan'
+POS_ROW_TYPE_BADPRICE           = 'badprice'
+POS_ROW_TYPE_ADJUST_PRICE       = 'adjust_price'
+POS_ROW_TYPE_VOID_LINE          = 'void_line'
+POS_ROW_TYPE_VOID_TXN           = 'void_txn'
+POS_ROW_TYPE_SUSPEND            = 'suspend'
+POS_ROW_TYPE_RESUME             = 'resume'
+POS_ROW_TYPE_TENDER             = 'tender'
+POS_ROW_TYPE_CHANGE_BACK        = 'change_back'
+
+POS_ROW_TYPE = OrderedDict([
+    (POS_ROW_TYPE_SET_CUSTOMER,         "set customer"),
+    (POS_ROW_TYPE_SWAP_CUSTOMER,        "swap customer"),
+    (POS_ROW_TYPE_DEL_CUSTOMER,         "del customer"),
+    (POS_ROW_TYPE_SELL,                 "sell"),
+    (POS_ROW_TYPE_OPEN_RING,            "open ring"),
+    (POS_ROW_TYPE_BADSCAN,              "bad scan"),
+    (POS_ROW_TYPE_BADPRICE,             "bad price"),
+    (POS_ROW_TYPE_ADJUST_PRICE,         "adjust price"),
+    (POS_ROW_TYPE_VOID_LINE,            "void line"),
+    (POS_ROW_TYPE_VOID_TXN,             "void txn"),
+    (POS_ROW_TYPE_SUSPEND,              "suspend"),
+    (POS_ROW_TYPE_RESUME,               "resume"),
+    (POS_ROW_TYPE_TENDER,               "tender"),
+    (POS_ROW_TYPE_CHANGE_BACK,          "change back"),
+])
+
+
+########################################
+# Price Type
+########################################
+
 PRICE_TYPE_REGULAR              = 0
 PRICE_TYPE_TPR                  = 1
 PRICE_TYPE_SALE                 = 2
@@ -342,6 +475,10 @@ PRICE_TYPE = {
     }
 
 
+########################################
+# Purchase Batch Mode
+########################################
+
 PURCHASE_BATCH_MODE_ORDERING            = 10
 PURCHASE_BATCH_MODE_RECEIVING           = 20
 PURCHASE_BATCH_MODE_COSTING             = 30
@@ -353,6 +490,10 @@ PURCHASE_BATCH_MODE = {
 }
 
 
+########################################
+# Purchase Credit Status
+########################################
+
 PURCHASE_CREDIT_STATUS_NEW              = 10
 PURCHASE_CREDIT_STATUS_SUBMITTED        = 20
 PURCHASE_CREDIT_STATUS_SATISFIED        = 30
@@ -366,6 +507,10 @@ PURCHASE_CREDIT_STATUS = {
 }
 
 
+########################################
+# Purchase Status
+########################################
+
 PURCHASE_STATUS_NEW             = 1 # TODO: is this needed?
 PURCHASE_STATUS_ORDERED         = 10
 PURCHASE_STATUS_RECEIVED        = 20
@@ -381,6 +526,10 @@ PURCHASE_STATUS = {
 }
 
 
+########################################
+# Tempmon Appliance Type
+########################################
+
 TEMPMON_APPLIANCE_TYPE_COOLER           = 1
 TEMPMON_APPLIANCE_TYPE_FREEZER          = 2
 
@@ -390,6 +539,10 @@ TEMPMON_APPLIANCE_TYPE = {
 }
 
 
+########################################
+# Tempmon Disk Type
+########################################
+
 TEMPMON_DISK_TYPE_SDCARD                = 1
 TEMPMON_DISK_TYPE_USB                   = 2
 
@@ -399,6 +552,10 @@ TEMPMON_DISK_TYPE = {
 }
 
 
+########################################
+# Tempmon Probe Status
+########################################
+
 TEMPMON_PROBE_STATUS_GOOD_TEMP          = 1
 TEMPMON_PROBE_STATUS_LOW_TEMP           = 2
 TEMPMON_PROBE_STATUS_HIGH_TEMP          = 3
@@ -420,6 +577,10 @@ TEMPMON_PROBE_STATUS = {
 }
 
 
+########################################
+# Unit of Measure
+########################################
+
 # These values are taken from the SIL standard.
 UNIT_OF_MEASURE_NONE                    = '00'
 UNIT_OF_MEASURE_EACH                    = '01'
@@ -590,6 +751,10 @@ UNIT_OF_MEASURE = {
     }
 
 
+########################################
+# Upgrade Status
+########################################
+
 UPGRADE_STATUS_PENDING          = 1
 UPGRADE_STATUS_EXECUTING        = 2
 UPGRADE_STATUS_SUCCEEDED        = 3
@@ -603,6 +768,10 @@ UPGRADE_STATUS = {
 }
 
 
+########################################
+# User Event
+########################################
+
 USER_EVENT_LOGIN                = 1
 USER_EVENT_LOGOUT               = 2
 USER_EVENT_BECOME_ROOT          = 3
@@ -616,6 +785,10 @@ USER_EVENT = {
 }
 
 
+########################################
+# Employee Status
+########################################
+
 EMPLOYEE_STATUS_CURRENT         = 1
 EMPLOYEE_STATUS_FORMER          = 2
 
@@ -638,6 +811,10 @@ EMPLOYEE_STATUS = {
 #     }
 
 
+########################################
+# Trainwreck System
+########################################
+
 TRAINWRECK_SYSTEM_COREPOS       = 'corepos'
 TRAINWRECK_SYSTEM_LOCSMS        = 'locsms'
 
