@@ -75,9 +75,10 @@ class FeatureHandler(GenericHandler):
         return Schema(**kwargs)
 
     def get_defaults(self):
+        app_title = self.app.get_title()
         return {
-            'app_prefix': self.config.app_title().lower(),
-            'app_cap_prefix': self.config.app_title(),
+            'app_prefix': app_title.lower(),
+            'app_cap_prefix': app_title,
         }
 
     def do_generate(self, feature, **kwargs):
