@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,14 +24,9 @@
 Global Product Code
 """
 
-from __future__ import unicode_literals, absolute_import
-
-import six
-
 from rattail import barcodes
 
 
-@six.python_2_unicode_compatible
 class GPC(object):
     """
     Class to abstract the details of Global Product Code data.  Examples of
@@ -70,13 +65,13 @@ class GPC(object):
     """
 
     def __init__(self, value, calc_check_digit=False):
-        value = six.text_type(value)
+        value = str(value)
 
         if calc_check_digit == 'auto':
             calc_check_digit = 'upc' if len(value) < 12 else False
 
         if calc_check_digit is True or calc_check_digit == 'upc':
-            value += six.text_type(barcodes.upc_check_digit(value))
+            value += str(barcodes.upc_check_digit(value))
 
         self.value = int(value)
 
@@ -150,7 +145,7 @@ class GPC(object):
         Note that in this case the ``data_str`` value is the same as was
         originally provided to the constructor, but that isn't always the case.
         """
-        return six.text_type(int(self))[:-1]
+        return str(int(self))[:-1]
 
     @property
     def data_length(self):
@@ -172,5 +167,5 @@ class GPC(object):
         Returns the UPC as a somewhat more human-readable string.  Basically
         that just means the check digit is distinguished by a hyphen.
         """
-        upc = six.text_type(self)
+        upc = str(self)
         return "{0}-{1}".format(upc[:-1], upc[-1])
