@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,11 +24,14 @@
 Data Importing Framework
 """
 
-from __future__ import unicode_literals, absolute_import
-
 from .importers import Importer, FromCSV, FromQuery, FromDjango, BatchImporter, BulkImporter
-from .sqlalchemy import FromSQLAlchemy, ToSQLAlchemy
-from .postgresql import BulkToPostgreSQL
-from .handlers import ImportHandler, BulkImportHandler, FromSQLAlchemyHandler, ToSQLAlchemyHandler
-from .rattail import FromRattailHandler, ToRattailHandler
-from . import model
+from .handlers import ImportHandler, BulkImportHandler
+
+try:
+    from .sqlalchemy import FromSQLAlchemy, ToSQLAlchemy
+    from .postgresql import BulkToPostgreSQL
+    from .handlers import FromSQLAlchemyHandler, ToSQLAlchemyHandler
+    from .rattail import FromRattailHandler, ToRattailHandler
+    from . import model
+except ImportError:
+    pass # sqlalchemy not installed
