@@ -95,7 +95,7 @@ class FromRattailToCSV(FromSQLAlchemyToCSVMixin, FromRattailHandler, ToCSVHandle
 
     @property
     def host_title(self):
-        return self.config.node_title()
+        return self.app.get_node_title()
 
     def get_model(self):
         if self.config:
