@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,22 +24,18 @@
 Import Handlers
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import os
 import sys
 import shutil
 import tempfile
 import logging
+from collections import OrderedDict
 
-import six
 import humanize
-import sqlalchemy as sa
+import markupsafe
 
-from rattail.core import get_uuid
-from rattail.time import make_utc
-from rattail.util import OrderedDict, get_object_spec, progress_loop
-from rattail.mail import send_email
+from rattail.util import get_object_spec
+from rattail.exceptions import ConfigurationError
 
 
 log = logging.getLogger(__name__)
@@ -97,12 +93,19 @@ class ImportHandler(object):
         if self.config:
             self.app = self.config.get_app()
             self.enum = self.config.get_enum()
-            self.model = self.config.get_model()
-        self.importers = self.get_importers()
-        self.extra_importer_kwargs = kwargs.pop('extra_importer_kwargs', {})
+            try:
+                self.model = self.app.model
+            except ImportError:
+                pass
+
+        # nb. must assign extra attrs before we get_importers() since
+        # attrs may need to affect that behavior
         for key, value in kwargs.items():
             setattr(self, key, value)
 
+        self.importers = self.get_importers()
+        self.extra_importer_kwargs = kwargs.pop('extra_importer_kwargs', {})
+
     @classmethod
     def get_key(cls):
         return 'to_{}.from_{}.{}'.format(cls.local_key, cls.host_key, cls.direction)
@@ -195,7 +198,7 @@ class ImportHandler(object):
         from rattail.db import Session
 
         session = Session()
-        model = self.config.get_model()
+        model = self.app.model
         metadata = sa.MetaData(schema='batch', bind=session.bind)
         user = session.merge(kwargs['runas_user'])
         handler_spec = self.config.get('rattail.batch', 'importer.handler',
@@ -215,7 +218,7 @@ class ImportHandler(object):
                 importer._handler_key = key
 
                 batch = model.ImporterBatch()
-                batch.uuid = get_uuid()
+                batch.uuid = self.app.make_uuid()
                 batch.created_by = user
                 batch.batch_handler_spec = handler_spec
                 batch.import_handler_spec = get_object_spec(self)
@@ -292,7 +295,7 @@ class ImportHandler(object):
         return row_table
 
     def populate_row_table(self, session, importer, batch, row_table):
-        importer.now = make_utc(tzinfo=True)
+        importer.now = self.app.make_utc(tzinfo=True)
         importer.setup()
 
         # obtain host data
@@ -319,14 +322,14 @@ class ImportHandler(object):
         def delete(key, i):
             cached = importer.cached_local_data.pop(key)
             local_data = cached['data']
-            local_data['_object_str'] = six.text_type(cached['object'])
+            local_data['_object_str'] = str(cached['object'])
             sequence = batch.rowcount + 1
             self.make_batch_row(session, importer, row_table, sequence, None, local_data,
                                 status_code=self.enum.IMPORTER_BATCH_ROW_STATUS_DELETE)
             batch.rowcount += 1
 
-        progress_loop(delete, sorted(deleting), self.progress,
-                      message="Deleting {} data".format(importer.model_name))
+        self.app.progress_loop(delete, sorted(deleting), self.progress,
+                               message=f"Deleting {importer.model_name} data")
 
     def _populate_create_update(self, session, importer, batch, row_table, data):
 
@@ -360,12 +363,12 @@ class ImportHandler(object):
                                     status_code=status_code, status_text=status_text)
                 batch.rowcount += 1
 
-        progress_loop(record, data, self.progress,
-                      message="Populating batch for {}".format(importer._handler_key))
+        self.app.progress_loop(record, data, self.progress,
+                               message=f"Populating batch for {importer._handler_key}")
 
     def make_batch_row(self, session, importer, row_table, sequence, host_data, local_data, status_code=None, status_text=None):
         values = {
-            'uuid': get_uuid(),
+            'uuid': self.app.make_uuid(),
             'sequence': sequence,
             'object_str': '',
             'status_code': status_code,
@@ -376,13 +379,13 @@ class ImportHandler(object):
             if '_object_str' in host_data:
                 values['object_str'] = host_data['_object_str']
             elif '_host_object' in host_data:
-                values['object_str'] = six.text_type(host_data['_host_object'])
-            values['object_key'] = ','.join([six.text_type(host_data[f]) for f in importer.key])
+                values['object_str'] = str(host_data['_host_object'])
+            values['object_key'] = ','.join([str(host_data[f]) for f in importer.key])
         elif local_data:
             if '_object_str' in local_data:
                 values['object_str'] = local_data['_object_str']
             elif '_object' in local_data:
-                values['object_str'] = six.text_type(local_data['_object'])
+                values['object_str'] = str(local_data['_object'])
             values['object_key'] = ','.join([local_data[f] for f in importer.key])
 
         for field in importer.fields:
@@ -407,7 +410,7 @@ class ImportHandler(object):
            attribute after this method completes.  This would be a dictionary
            whose keys are model names and values are the importer instances.
         """
-        self.import_began = make_utc(tzinfo=True)
+        self.import_began = self.app.make_utc(tzinfo=True)
         retain_used_importers = kwargs.pop('retain_used_importers', False)
         if 'dry_run' in kwargs:
             self.dry_run = kwargs['dry_run']
@@ -521,6 +524,23 @@ class ImportHandler(object):
     def commit_local_transaction(self):
         pass
 
+    def get_cmd(self, ignore_errors=False):
+        handler_key = self.get_key()
+
+        cmd = self.config.getlist('rattail.importing', f'{handler_key}.cmd')
+        if not cmd or len(cmd) != 2:
+            cmd = self.config.getlist('rattail.importing',
+                                      f'{handler_key}.default_cmd')
+
+            if not cmd or len(cmd) != 2:
+                msg = (f"Missing or invalid config; please set '{handler_key}.default_cmd' in the "
+                       "[rattail.importing] section of your config file")
+                if ignore_errors:
+                    return
+                raise ConfigurationError(msg)
+
+        return cmd
+
     def process_changes(self, changes):
         """
         This method is called any time changes occur, regardless of whether the
@@ -534,7 +554,7 @@ class ImportHandler(object):
         if not self.warnings:
             return
 
-        now = make_utc(tzinfo=True)
+        now = self.app.make_utc(tzinfo=True)
         data = {
             'local_title': self.local_title,
             'host_title': self.host_title,
@@ -547,19 +567,19 @@ class ImportHandler(object):
             'max_display': self.diff_max_display,
         }
 
-        command = getattr(self, 'command', None)
+        command = self.get_cmd(ignore_errors=True)
         if command:
-            data['command'] = '{} {}'.format(command.parent.name, command.name)
-        else:
-            data['command'] = None
+            command, subcommand = command
 
         if command:
-            key = '{}_{}_updates'.format(command.parent.name, command.name)
+            data['command'] = f'{command} {subcommand}'
+            key = f'{command}_{subcommand}_updates'
             key = key.replace('-', '_')
         else:
+            data['command'] = None
             key = 'rattail_import_updates'
 
-        send_email(self.config, key, fallback_key='rattail_import_updates', data=data)
+        self.app.send_email(key, fallback_key='rattail_import_updates', data=data)
         log.info("{} -> {}: warning email was sent".format(self.host_title, self.local_title))
 
 
@@ -573,7 +593,7 @@ class BulkImportHandler(ImportHandler):
         Import all data for the given importer/model keys.
         """
         # TODO: still need to refactor much of this so can share with parent class
-        self.import_began = make_utc(tzinfo=True)
+        self.import_began = self.app.make_utc(tzinfo=True)
         if 'dry_run' in kwargs:
             self.dry_run = kwargs['dry_run']
         self.progress = kwargs.pop('progress', getattr(self, 'progress', None))
@@ -615,69 +635,6 @@ class BulkImportHandler(ImportHandler):
         return changes
 
 
-class FromSQLAlchemyHandler(ImportHandler):
-    """
-    Handler for imports for which the host data source is represented by a
-    SQLAlchemy engine and ORM.
-    """
-    host_session = None
-
-    def make_host_session(self):
-        """
-        Subclasses must override this to define the host database connection.
-        """
-        raise NotImplementedError
-
-    def get_importer_kwargs(self, key, **kwargs):
-        kwargs = super(FromSQLAlchemyHandler, self).get_importer_kwargs(key, **kwargs)
-        kwargs.setdefault('host_session', self.host_session)
-        return kwargs
-
-    def begin_host_transaction(self):
-        self.host_session = self.make_host_session()
-
-    def rollback_host_transaction(self):
-        self.host_session.rollback()
-        self.host_session.close()
-        self.host_session = None
-
-    def commit_host_transaction(self):
-        self.host_session.commit()
-        self.host_session.close()
-        self.host_session = None
-
-
-class ToSQLAlchemyHandler(ImportHandler):
-    """
-    Handler for imports which target a SQLAlchemy ORM on the local side.
-    """
-    session = None
-
-    def make_session(self):
-        """
-        Subclasses must override this to define the local database connection.
-        """
-        raise NotImplementedError
-
-    def get_importer_kwargs(self, key, **kwargs):
-        kwargs = super(ToSQLAlchemyHandler, self).get_importer_kwargs(key, **kwargs)
-        kwargs.setdefault('session', self.session)
-        return kwargs
-
-    def begin_local_transaction(self):
-        self.session = self.make_session()
-
-    def rollback_local_transaction(self):
-        self.session.rollback()
-        self.session.close()
-        self.session = None
-
-    def commit_local_transaction(self):
-        self.session.commit()
-        self.session.close()
-        self.session = None
-
-
 class FromFileHandler(ImportHandler):
     """
     Handler for imports whose data comes from file(s) on the host side.
@@ -760,7 +717,7 @@ class RecordRenderer(object):
         label = self.get_label(record)
         url = self.get_url(record)
         if url:
-            return '<a href="{}">{}</a>'.format(url, label)
+            return markupsafe.Markup(f'<a href="{url}">{label}</a>')
         return label
 
     def get_label(self, record):
@@ -769,7 +726,7 @@ class RecordRenderer(object):
         return label(record)
 
     def label(self, record):
-        return six.text_type(record)
+        return str(record)
 
     def get_url(self, record):
         """
@@ -785,11 +742,79 @@ class RecordRenderer(object):
         Fetch / generate a URL for the given data record.
         """
         if hasattr(record, 'uuid'):
-            url = self.config.get('tailbone', 'url')
+            url = self.config.base_url()
             if url:
-                url = url.rstrip('/')
                 name = '{}s'.format(record.__class__.__name__.lower())
                 if name == 'persons': # FIXME, obviously this is a hack
                     name = 'people'
                 url = '{}/{}/{{uuid}}'.format(url, name)
                 return url.format(uuid=record.uuid)
+
+
+try:
+    import sqlalchemy as sa
+except ImportError:
+    pass
+else:
+
+    class FromSQLAlchemyHandler(ImportHandler):
+        """
+        Handler for imports for which the host data source is represented by a
+        SQLAlchemy engine and ORM.
+        """
+        host_session = None
+
+        def make_host_session(self):
+            """
+            Subclasses must override this to define the host database connection.
+            """
+            raise NotImplementedError
+
+        def get_importer_kwargs(self, key, **kwargs):
+            kwargs = super(FromSQLAlchemyHandler, self).get_importer_kwargs(key, **kwargs)
+            kwargs.setdefault('host_session', self.host_session)
+            return kwargs
+
+        def begin_host_transaction(self):
+            self.host_session = self.make_host_session()
+
+        def rollback_host_transaction(self):
+            self.host_session.rollback()
+            self.host_session.close()
+            self.host_session = None
+
+        def commit_host_transaction(self):
+            self.host_session.commit()
+            self.host_session.close()
+            self.host_session = None
+
+
+    class ToSQLAlchemyHandler(ImportHandler):
+        """
+        Handler for imports which target a SQLAlchemy ORM on the local side.
+        """
+        session = None
+
+        def make_session(self):
+            """
+            Subclasses must override this to define the local database connection.
+            """
+            raise NotImplementedError
+
+        def get_importer_kwargs(self, key, **kwargs):
+            kwargs = super(ToSQLAlchemyHandler, self).get_importer_kwargs(key, **kwargs)
+            kwargs.setdefault('session', self.session)
+            return kwargs
+
+        def begin_local_transaction(self):
+            self.session = self.make_session()
+
+        def rollback_local_transaction(self):
+            self.session.rollback()
+            self.session.close()
+            self.session = None
+
+        def commit_local_transaction(self):
+            self.session.commit()
+            self.session.close()
+            self.session = None
