@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,17 +24,11 @@
 Data Importers
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import datetime
 import logging
+from collections import OrderedDict
 
-import six
-
-from rattail.db import cache
-from rattail.db.util import QuerySequence
-from rattail.time import make_utc
-from rattail.util import data_diffs, progress_loop, OrderedDict
+from rattail.util import data_diffs
 from rattail.csvutil import UnicodeDictReader
 
 
@@ -130,7 +124,10 @@ class Importer(object):
         self.config = config
         self.app = config.get_app() if config else None
         self.enum = config.get_enum() if config else None
-        self.model = config.get_model() if config else None
+        try:
+            self.model = self.app.model if config else None
+        except ImportError:
+            pass
         self.model_class = kwargs.pop('model_class', self.get_model_class())
         if key is not None:
             self.key = key
@@ -140,7 +137,7 @@ class Importer(object):
             self.exclude_fields(*exclude_fields)
         self.fuzzy_fields = fuzzy_fields or []
         self.fuzz_factor = fuzz_factor
-        if isinstance(self.key, six.string_types):
+        if isinstance(self.key, str):
             self.key = (self.key,)
         if self.key:
             for field in self.key:
@@ -203,38 +200,49 @@ class Importer(object):
         now = self.app.localtime()
         today = now.date()
 
-        # start time defaults to midnight this morning, unless specified
-        if not hasattr(self, 'start_time') or not self.start_time:
-            if not hasattr(self, 'start_date') or not self.start_date:
-                if (hasattr(self, 'args')
-                    and hasattr(self.args, 'start_date')
-                    and self.args.start_date):
-                    self.start_date = self.args.start_date
-                else:
-                    self.start_date = today
+        # start date is empty by default, unless specified
+        start_date = None
+        if hasattr(self, 'start_date'):
+            start_date = self.start_date
+        if not start_date and hasattr(self, 'args') and hasattr(self.args, 'start_date'):
+            start_date = self.args.start_date
+        self.start_date = start_date
+
+        # start time defaults to 12am midnight of start date, if applicable
+        start_time = None
+        if hasattr(self, 'start_time'):
+            start_time = self.start_time
+        if not start_time and self.start_date:
             start_time = datetime.datetime.combine(self.start_date,
                                                    datetime.time(0))
-            self.start_time = self.app.localtime(start_time)
+            start_time = self.app.localtime(start_time)
+        self.start_time = start_time
 
-        # end_time defaults to midnight tonight, unless specified
-        if not hasattr(self, 'end_time') or not self.end_time:
-            if not hasattr(self, 'end_date') or not self.end_date:
-                if (hasattr(self, 'args')
-                    and hasattr(self.args, 'end_date')
-                    and self.args.end_date):
-                    self.end_date = self.args.end_date
-                else:
-                    self.end_date = today
+        # end date is empty by default, unless specified
+        end_date = None
+        if hasattr(self, 'end_date'):
+            end_date = self.end_date
+        if not end_date and hasattr(self, 'args') and hasattr(self.args, 'end_date'):
+            end_date = self.args.end_date
+        self.end_date = end_date
+
+        # end time defaults to 12am midnight of *day after* end date, unless specified
+        end_time = None
+        if hasattr(self, 'end_time'):
+            end_time = self.end_time
+        if not end_time and self.end_date:
             end_date = self.end_date + datetime.timedelta(days=1)
             end_time = datetime.datetime.combine(end_date, datetime.time(0))
-            self.end_time = self.app.localtime(end_time)
+            end_time = self.app.localtime(end_time)
+        self.end_time = end_time
 
         # and some commands use --year instead of date range
-        if not hasattr(self, 'year') or not self.year:
-            if hasattr(self, 'args') and hasattr(self.args, 'year'):
-                self.year = self.args.year
-            else:
-                self.year = None
+        year = None
+        if hasattr(self, 'year'):
+            year = self.year
+        if not year and hasattr(self, 'args') and hasattr(self.args, 'year'):
+            year = self.args.year
+        self.year = year
 
     def setup(self):
         """
@@ -253,7 +261,7 @@ class Importer(object):
 
     def progress_loop(self, func, items, factory=None, **kwargs):
         factory = factory or self.progress
-        return progress_loop(func, items, factory, **kwargs)
+        return self.app.progress_loop(func, items, factory, **kwargs)
 
     def unique_data(self, host_data, warn=True):
         # Prune duplicate keys from host/source data.  This is for the sake of
@@ -276,7 +284,7 @@ class Importer(object):
         of where data is coming from or where it's headed.  Note that this
         method handles deletions as well as adds/updates.
         """
-        self.now = now or make_utc(tzinfo=True)
+        self.now = now or self.app.make_utc(tzinfo=True)
         if kwargs:
             self._setup(**kwargs)
         self.setup()
@@ -313,6 +321,32 @@ class Importer(object):
         self.teardown()
         return created, updated, deleted
 
+    # TODO: this should probably be used with datasync where possible?
+    def import_single_object(self, host_object, **kwargs):
+        """
+        Import a single object from host.  This is meant primarily for
+        use with scripts etc. and is not part of a "normal" (full)
+        import run.
+        """
+        host_data = self.normalize_host_object(host_object)
+        key = self.get_key(host_data)
+
+        local_object = self.get_local_object(key)
+        if local_object:
+            if self.allow_update:
+
+                # update local object only if data differs
+                local_data = self.normalize_local_object(local_object)
+                if self.data_diffs(local_data, host_data) and self.allow_update:
+                    local_object = self.update_object(local_object, host_data, local_data)
+
+            return local_object
+
+        elif self.allow_create:
+
+            # create new local object
+            return self.create_object(key, host_data)
+
     def _import_create_update(self, data, created=None, updated=None):
         """
         Import the given data; create and/or update records as needed.
@@ -489,6 +523,16 @@ class Importer(object):
                     log.warning("max of {} *total changes* has been reached; stopping now".format(self.max_total))
                     raise ImportLimitReached()
 
+            # flush changes every so often
+            flush = False
+            if not self.batch_size or self.batch_size == 1:
+                # flush every time if no meaningful batch size
+                flush = True
+            elif deleted and len(deleted) % self.batch_size == 0:
+                flush = True
+            if flush:
+                self.flush_delete()
+
         try:
             self.progress_loop(delete, sorted(deleting),
                                message="Deleting {} data".format(self.model_name))
@@ -497,6 +541,12 @@ class Importer(object):
 
         return deleted
 
+    def flush_delete(self):
+        """
+        Perform any steps necessary to "flush" the create/update changes which
+        have occurred thus far in the import.
+        """
+
     def get_key(self, data):
         """
         Return the key value for the given data dict.
@@ -571,8 +621,9 @@ class Importer(object):
 
         def cache(obj, i):
             data = self.normalize_local_object(obj)
-            key = self.get_key(data)
-            cached[key] = {'object': obj, 'data': data}
+            if data:
+                key = self.get_key(data)
+                cached[key] = {'object': obj, 'data': data}
 
         self.progress_loop(cache, objects,
                            message="Reading {} data from {}".format(
@@ -663,7 +714,7 @@ class Importer(object):
         if not session:
             session = self.session
         kwargs.setdefault('progress', self.progress)
-        return cache.cache_model(session, model, **kwargs)
+        return self.app.cache_model(session, model, **kwargs)
 
     def data_diffs(self, local_data, host_data):
         """
@@ -783,13 +834,8 @@ class FromCSV(Importer):
                 self.source_data_path = self.args.source_csv
 
     def get_host_objects(self):
-        if six.PY3:
-            source_csv_file = open(self.source_data_path, 'rt', encoding='latin_1')
-            reader = UnicodeDictReader(source_csv_file)
-        else:
-            source_csv_file = open(self.source_data_path, 'rb')
-            reader = UnicodeDictReader(source_csv_file, 'latin_1')
-
+        source_csv_file = open(self.source_data_path, 'rt', encoding='latin_1')
+        reader = UnicodeDictReader(source_csv_file)
         objects = list(reader)
         source_csv_file.close()
         return objects
@@ -812,7 +858,12 @@ class FromQuery(Importer):
         """
         Returns (raw) query results as a sequence.
         """
-        return QuerySequence(self.query())
+        from rattail.db.util import QuerySequence
+
+        query = self.query()
+        if hasattr(self, 'sorted_query'):
+            query = self.sorted_query(query)
+        return QuerySequence(query)
 
 
 class FromDjango(FromQuery):
@@ -839,7 +890,7 @@ class BatchImporter(Importer):
         if host_data is not None:
             raise ValueError("User-provided host data is not supported")
 
-        self.now = now or make_utc(tzinfo=True)
+        self.now = now or self.app.make_utc(tzinfo=True)
         if kwargs:
             self._setup(**kwargs)
         self.setup()
@@ -898,7 +949,7 @@ class BulkImporter(Importer):
     """
 
     def import_data(self, host_data=None, now=None, **kwargs):
-        self.now = now or make_utc(tzinfo=True)
+        self.now = now or self.app.make_utc(tzinfo=True)
         if kwargs:
             self._setup(**kwargs)
         self.setup()
