@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -39,13 +39,12 @@ class FromRattailSelfToRattail(importing.FromRattailHandler, importing.ToSQLAlch
 
     @property
     def host_title(self):
-        title = self.config.node_title(default="Rattail")
-        return f"{title} (self)"
+        node_title = self.app.get_node_title()
+        return f"{node_title} (self)"
 
     @property
     def local_title(self):
-        title = self.config.node_title(default="Rattail")
-        return title
+        return self.app.get_node_title()
 
     def begin_local_transaction(self):
         self.session = self.host_session
