@@ -46,7 +46,7 @@ class FromRattailHandler(FromSQLAlchemyHandler):
 
     @property
     def host_title(self):
-        return self.config.app_title(default="Rattail")
+        return self.app.get_title()
 
     def make_host_session(self):
         return self.app.make_session()
@@ -61,7 +61,7 @@ class ToRattailHandler(ToSQLAlchemyHandler):
 
     @property
     def local_title(self):
-        return self.config.app_title(default="Rattail")
+        return self.app.get_title()
 
     def make_session(self):
         kwargs = {}
@@ -187,14 +187,19 @@ class FromRattailToRattailImport(FromRattailToRattailBase, FromRattailHandler, T
 
     @property
     def host_title(self):
-        return "{} ({})".format(self.config.app_title(default="Rattail"), self.dbkey)
+        app_title = self.app.get_title()
+        return f"{app_title} ({self.dbkey})"
 
     @property
     def local_title(self):
-        return self.config.node_title(default="Rattail (local)")
+        app_title = self.app.get_title()
+        node_title = self.app.get_node_title()
+        if node_title != app_title:
+            return node_title
+        return f"{app_title} (local)"
 
     def make_host_session(self):
-        return self.app.make_session(bind=self.config.rattail_engines[self.dbkey])
+        return self.app.make_session(bind=self.config.appdb_engines[self.dbkey])
 
 
 class FromRattailToRattailExport(FromRattailToRattailBase, FromRattailHandler, ToRattailHandler):
@@ -211,14 +216,15 @@ class FromRattailToRattailExport(FromRattailToRattailBase, FromRattailHandler, T
 
     @property
     def host_title(self):
-        return self.config.node_title()
+        return self.app.get_node_title()
 
     @property
     def local_title(self):
-        return "{} ({})".format(self.config.app_title(default="Rattail"), self.dbkey)
+        app_title = self.app.get_title()
+        return f"{app_title} ({self.dbkey})"
 
     def make_session(self):
-        return self.app.make_session(bind=self.config.rattail_engines[self.dbkey])
+        return self.app.make_session(bind=self.config.appdb_engines[self.dbkey])
 
 
 class FromRattail(FromSQLAlchemySameToSame):
