@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,21 +24,15 @@
 Sample -> Rattail data import
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import csv
 import os
 import datetime
 import decimal
 import logging
-
-import six
+from collections import OrderedDict
 
 from rattail import importing
-from rattail.util import OrderedDict
-from rattail.csvutil import UnicodeDictReader
 from rattail.files import resource_path
-from rattail.config import parse_bool
 
 
 log = logging.getLogger(__name__)
@@ -97,11 +91,6 @@ class FromSample(importing.Importer):
         """
         Return all data rows from CSV file, as returned by CSV parser.
         """
-        if six.PY2:
-            with open(self.data_path, 'rb') as csv_file:
-                reader = UnicodeDictReader(csv_file)
-                return list(reader)
-
         with open(self.data_path, 'rt') as csv_file:
             reader = csv.DictReader(csv_file)
             return list(reader)
@@ -131,12 +120,12 @@ class UserImporter(FromSample, importing.model.AdminUserImporter):
 
     def normalize_host_object(self, row):
         data = dict(row)
-        data['active'] = parse_bool(data['active']) if data['active'] else None
-        data['admin'] = parse_bool(data['admin']) if data['admin'] else None
+        data['active'] = self.config.parse_bool(data['active']) if data['active'] else None
+        data['admin'] = self.config.parse_bool(data['admin']) if data['admin'] else None
         return data
 
     def data_diffs(self, local_data, host_data):
-        diffs = super(UserImporter, self).data_diffs(local_data, host_data)
+        diffs = super().data_diffs(local_data, host_data)
         if 'plain_password' in diffs:
             diffs.remove('plain_password')
         return diffs
@@ -157,7 +146,7 @@ class EmployeeImporter(FromSample, importing.model.EmployeeImporter):
         data = dict(row)
         data['id'] = int(data['id']) if data['id'] else None
         data['status'] = int(data['status']) if data['status'] else None
-        data['full_time'] = parse_bool(data['full_time'])
+        data['full_time'] = self.config.parse_bool(data['full_time'])
         if data['full_time_start']:
             data['full_time_start'] = datetime.datetime.strptime(data['full_time_start'], '%Y-%m-%d').date()
         else:
@@ -174,8 +163,8 @@ class CustomerImporter(FromSample, importing.model.CustomerImporter):
         data = dict(row)
         data['number'] = int(data['number']) if data['number'] else None
         data['email_preference'] = int(data['email_preference']) if data['email_preference'] else None
-        data['active_in_pos'] = parse_bool(data['active_in_pos'])
-        data['active_in_pos_sticky'] = parse_bool(data['active_in_pos_sticky'])
+        data['active_in_pos'] = self.config.parse_bool(data['active_in_pos'])
+        data['active_in_pos_sticky'] = self.config.parse_bool(data['active_in_pos_sticky'])
         return data
 
 
