@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2022 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,10 +24,7 @@
 Data Importers for SQLAlchemy
 """
 
-from __future__ import unicode_literals, absolute_import
-
 from sqlalchemy import orm
-from sqlalchemy.orm.exc import NoResultFound
 
 from rattail.importing import Importer, FromQuery
 
@@ -50,6 +47,17 @@ class FromSQLAlchemy(FromQuery):
         """
         return self.host_session.query(self.host_model_class)
 
+    def sorted_query(self, query):
+        if hasattr(query, 'order_by'):
+            columns = []
+            for field in self.key:
+                column = getattr(self.host_model_class, field, None)
+                if column:
+                    columns.append(column)
+            if columns:
+                query = query.order_by(*columns)
+        return query
+
 
 class ToSQLAlchemy(Importer):
     """
@@ -98,7 +106,7 @@ class ToSQLAlchemy(Importer):
             query = query.filter(getattr(self.model_class, k) == key[i])
         try:
             return query.one()
-        except NoResultFound:
+        except orm.exc.NoResultFound:
             pass
 
     def create_object(self, key, host_data):
@@ -154,6 +162,13 @@ class ToSQLAlchemy(Importer):
         """
         self.session.flush()
 
+    def flush_delete(self):
+        """
+        Flush the database session, to send SQL to the server for all changes
+        made thus far.
+        """
+        self.session.flush()
+
 
 class FromSQLAlchemySameToSame(FromSQLAlchemy):
     """
