@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -44,8 +44,8 @@ class FromRattailToRattailVersions(FromRattailHandler, ToRattailHandler):
 
     @property
     def local_title(self):
-        app_title = self.config.app_title(default="Rattail")
-        return "{} (versioning)".format(app_title)
+        app_title = self.app.get_title()
+        return f"{app_title} (versioning)"
 
     def get_importers(self):
         importers = OrderedDict()
@@ -105,14 +105,14 @@ class FromRattailToRattailVersions(FromRattailHandler, ToRattailHandler):
         return importers
 
     def begin_local_transaction(self):
-        super(FromRattailToRattailVersions, self).begin_local_transaction()
+        super().begin_local_transaction()
         self.uow = versioning_manager.unit_of_work(self.session)
         self.transaction = self.uow.create_transaction(self.session)
         if self.comment:
             self.transaction.meta = {'comment': self.comment}
 
     def get_importer_kwargs(self, key, **kwargs):
-        kwargs = super(FromRattailToRattailVersions, self).get_importer_kwargs(key, **kwargs)
+        kwargs = super().get_importer_kwargs(key, **kwargs)
         kwargs['transaction'] = self.transaction
         return kwargs
 
@@ -127,7 +127,7 @@ class VersionImporter(FromRattail, ToRattail):
 
     @property
     def simple_fields(self):
-        fields = super(VersionImporter, self).simple_fields
+        fields = super().simple_fields
         fields.remove('operation_type')
         fields.remove('transaction_id')
         fields.remove('end_transaction_id')
@@ -139,7 +139,7 @@ class VersionImporter(FromRattail, ToRattail):
                            .filter(self.model_class.operation_type != continuum.Operation.DELETE)
 
     def normalize_local_object(self, obj):
-        data = super(VersionImporter, self).normalize_local_object(obj)
+        data = super().normalize_local_object(obj)
         # cache local (version) object, for updating end_transaction_id
         # (note that this is called for host side as well, hence check)
         if isinstance(obj, self.model_class):
