@@ -270,9 +270,9 @@ class LuigiHandler(GenericHandler):
         # foribly recreate the connection pool for all overnight task
         # runs.  would be nice to be more selective but this seems to
         # work okay for now...
-        pool = self.config.rattail_engine.pool
+        pool = self.config.appdb_engine.pool
         pool.dispose()
-        self.config.rattail_engine.pool = pool.recreate()
+        self.config.appdb_engine.pool = pool.recreate()
 
         # run command in subprocess
         curdir = os.getcwd()
