@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,10 +24,8 @@
 Problem Reports Framework
 """
 
-from rattail.util import progress_loop
 
-
-class ProblemReport(object):
+class ProblemReport:
     """
     Base class for all problem reports.
 
@@ -46,12 +44,12 @@ class ProblemReport(object):
         self.dry_run = dry_run
         self.progress = progress
         self.app = self.config.get_app()
-        self.model = self.config.get_model()
+        self.model = self.app.model
         self.enum = self.config.get_enum()
 
     def progress_loop(self, func, items, factory=None, **kwargs):
         factory = factory or self.progress
-        return progress_loop(func, items, factory, **kwargs)
+        return self.app.progress_loop(func, items, factory, **kwargs)
 
     def cache_model(self, session, model, **kwargs):
         """
@@ -77,3 +75,11 @@ class ProblemReport(object):
         email template.
         """
         return kwargs
+
+    def make_email_attachments(self, context):
+        """
+        Optionally generate some attachment(s) for the report email.
+
+        :param context: Dict of context for the report email.  In
+           particular see ``context['problems']`` for main data.
+        """
