@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -28,12 +28,7 @@ import decimal
 import warnings
 import logging
 
-import sqlalchemy as sa
-from sqlalchemy import orm
-
 from rattail import pod
-from rattail.db import api
-from rattail.util import load_object
 from rattail.app import GenericHandler, MergeMixin
 from rattail.gpc import GPC
 from rattail.barcodes import upce_to_upca, upc_check_digit
@@ -159,7 +154,12 @@ class ProductsHandler(GenericHandler, MergeMixin):
         return self.config.getbool('rattail', 'products.units_only',
                                    default=False)
 
-    def locate_product_for_entry(self, session, entry, **kwargs):
+    def locate_product_for_entry(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            **kwargs):
         """
         This method aims to provide sane default logic for locating a
         :class:`~rattail.db.model.products.Product` record for the
@@ -191,6 +191,9 @@ class ProductsHandler(GenericHandler, MergeMixin):
            * ``vendor_code`` - :meth:`locate_product_for_vendor_code`
            * ``alt_code`` - :meth:`locate_product_for_alt_code`
 
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
@@ -198,37 +201,39 @@ class ProductsHandler(GenericHandler, MergeMixin):
         if not entry:
             return
 
-        # if entry is GPC then only look for that type of match
-        # TODO: is this really the expected behavior?  maybe not..
-        if isinstance(entry, GPC):
-            return self.locate_product_for_gpc(session, entry)
-
         # figure out which fields we should match on
         # TODO: let config declare default lookup_fields
-        lookup_fields = kwargs.get('lookup_fields', [
+        lookup_fields = kwargs.pop('lookup_fields', [
             'uuid',
             '_product_key_',
         ])
-        if kwargs.get('lookup_vendor_code'): # pragma: no cover
+        if kwargs.pop('lookup_vendor_code', False): # pragma: no cover
             warnings.warn("`lookup_vendor_code` param is deprecated, "
                           "please include 'vendor_code' in `lookup_fields` "
                           "instead", DeprecationWarning, stacklevel=2)
             if 'vendor_code' not in lookup_fields:
                 lookup_fields.append('vendor_code')
-        if kwargs.get('lookup_by_code'): # pragma: no cover
+        if kwargs.pop('lookup_by_code', False): # pragma: no cover
             warnings.warn("`lookup_by_code` param is deprecated, "
                           "please include 'alt_code' in `lookup_fields` "
                           "instead", DeprecationWarning, stacklevel=2)
             if 'alt_code' not in lookup_fields:
                 lookup_fields.append('alt_code')
 
+        kwargs['include_not_for_sale'] = include_not_for_sale
+
+        # if entry is GPC then only look for that type of match
+        if isinstance(entry, GPC):
+            return self.locate_product_for_gpc(session, entry, **kwargs)
+
         # try to locate product by uuid before other, more specific key
-        # if kwargs.get('try_uuid', True):
         if 'uuid' in lookup_fields:
             if isinstance(entry, str):
                 product = session.get(model.Product, entry)
-                if product:
-                    return product
+                # TODO: should we ever return deleted product?
+                if product and not product.deleted:
+                    if include_not_for_sale or not product.not_for_sale:
+                        return product
 
         lookups = {
             'uuid': None,
@@ -301,7 +306,13 @@ class ProductsHandler(GenericHandler, MergeMixin):
             if product:
                 return product
 
-    def locate_product_for_item_id(self, session, entry, **kwargs):
+    def locate_product_for_item_id(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            include_deleted=False,
+            **kwargs):
         """
         Locate the product which matches the given item ID.
 
@@ -315,10 +326,16 @@ class ProductsHandler(GenericHandler, MergeMixin):
 
         :param entry: Item ID value as string.
 
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :param include_deleted: Whether "deleted" products should ever
+           match (and be returned).
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
-        from rattail.db.api.products import get_product_by_item_id
+        from sqlalchemy import orm
 
         if not entry:
             return
@@ -326,10 +343,25 @@ class ProductsHandler(GenericHandler, MergeMixin):
         # assume entry is string
         entry = str(entry)
 
-        # do basic lookup
-        return get_product_by_item_id(session, entry)
+        model = self.model
+        products = session.query(model.Product)
+        if not include_deleted:
+            products = products.filter(model.Product.deleted == False)
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
 
-    def locate_product_for_scancode(self, session, entry, **kwargs):
+        try:
+            return products.filter(model.Product.item_id == entry).one()
+        except orm.exc.NoResultFound:
+            pass
+
+    def locate_product_for_scancode(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            include_deleted=False,
+            **kwargs):
         """
         Locate the product which matches the given scancode.
 
@@ -343,10 +375,16 @@ class ProductsHandler(GenericHandler, MergeMixin):
 
         :param entry: Scancode value as string.
 
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :param include_deleted: Whether "deleted" products should ever
+           match (and be returned).
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
-        from rattail.db.api.products import get_product_by_scancode
+        from sqlalchemy import orm
 
         if not entry:
             return
@@ -354,10 +392,24 @@ class ProductsHandler(GenericHandler, MergeMixin):
         # assume entry is string
         entry = str(entry)
 
-        # do basic lookup
-        return get_product_by_scancode(session, entry)
+        model = self.model
+        products = session.query(model.Product)
+        if not include_deleted:
+            products = products.filter(model.Product.deleted == False)
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
 
-    def locate_product_for_key(self, session, entry, product_key=None, **kwargs):
+        try:
+            return products.filter(model.Product.scancode == entry).one()
+        except orm.exc.NoResultFound:
+            pass
+
+    def locate_product_for_key(
+            self,
+            session,
+            entry,
+            product_key=None,
+            **kwargs):
         """
         Locate the product which matches the given key value.
 
@@ -387,22 +439,28 @@ class ProductsHandler(GenericHandler, MergeMixin):
         """
         # prefer caller-provided key over configured key
         if not product_key:
-            product_key = self.config.product_key()
+            product_key = self.app.get_product_key_field()
 
         product = None
 
         if product_key == 'upc':
-            product = self.locate_product_for_upc(session, entry)
+            product = self.locate_product_for_upc(session, entry, **kwargs)
 
         elif product_key == 'item_id':
-            product = self.locate_product_for_item_id(session, entry)
+            product = self.locate_product_for_item_id(session, entry, **kwargs)
 
         elif product_key == 'scancode':
-            product = self.locate_product_for_scancode(session, entry)
+            product = self.locate_product_for_scancode(session, entry, **kwargs)
 
         return product
 
-    def locate_product_for_gpc(self, session, gpc, **kwargs):
+    def locate_product_for_gpc(
+            self,
+            session,
+            gpc,
+            type2_lookup=None,
+            include_not_for_sale=False,
+            **kwargs):
         """
         Try to locate a product for the given GPC value.
 
@@ -410,9 +468,19 @@ class ProductsHandler(GenericHandler, MergeMixin):
 
         :param gpc: :class:`~rattail.gpc.GPC` instance to match on.
 
+        :param type2_lookup: Optional boolean indicating whether a
+           "type 2" UPC lookup should be attempted, if applicable.  By
+           default, config will determine whether a type 2 lookup may
+           be attempted.
+
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
+        from sqlalchemy import orm
+
         if not gpc:
             return
 
@@ -426,6 +494,9 @@ class ProductsHandler(GenericHandler, MergeMixin):
             #if not kwargs.get('include_deleted'):
             products = products.filter(model.Product.deleted == False)
 
+            if not include_not_for_sale:
+                products = products.filter(model.Product.not_for_sale == False)
+
             # TODO: maybe should just use first() instead?
             try:
                 return products.one()
@@ -438,17 +509,26 @@ class ProductsHandler(GenericHandler, MergeMixin):
             return product
 
         # maybe also try special search for "Type 2 UPC"
-        if gpc.type2_upc and self.convert_type2_for_gpc_lookup():
-            cleaned = self.make_gpc('002{}00000'.format(gpc.data_str[1:6]),
-                                    calc_check_digit='upc')
-            return lookup(cleaned)
+        if gpc.type2_upc:
+            if type2_lookup is None:
+                type2_lookup = self.convert_type2_for_gpc_lookup()
+            if type2_lookup:
+                cleaned = self.make_gpc('002{}00000'.format(gpc.data_str[1:6]),
+                                        calc_check_digit='upc')
+                return lookup(cleaned)
 
     def convert_type2_for_gpc_lookup(self):
         return self.config.getbool(
             'rattail', 'products.convert_type2_for_gpc_lookup',
             default=False)
 
-    def locate_product_for_vendor_code(self, session, entry, vendor=None, **kwargs):
+    def locate_product_for_vendor_code(
+            self,
+            session,
+            entry,
+            vendor=None,
+            include_not_for_sale=False,
+            **kwargs):
         """
         Locate the product which matches the given vendor code.
 
@@ -464,6 +544,9 @@ class ProductsHandler(GenericHandler, MergeMixin):
            in most cases you should specify the vendor when doing this
            type of lookup.
 
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
@@ -475,6 +558,9 @@ class ProductsHandler(GenericHandler, MergeMixin):
         if vendor:
             query = query.filter(model.ProductCost.vendor == vendor)
 
+        if not include_not_for_sale:
+            query = query.filter(model.Product.not_for_sale == False)
+
         if not kwargs.get('include_deleted'):
             query = query.filter(model.Product.deleted == False)
 
@@ -492,7 +578,13 @@ class ProductsHandler(GenericHandler, MergeMixin):
 
             return products[0]
 
-    def locate_product_for_alt_code(self, session, entry, **kwargs):
+    def locate_product_for_alt_code(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            include_deleted=False,
+            **kwargs):
         """
         Locate the product which matches the given alternate code.
 
@@ -503,12 +595,190 @@ class ProductsHandler(GenericHandler, MergeMixin):
 
         :param entry: Alternate code value as string.
 
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :param include_deleted: Whether "deleted" products should ever
+           match (and be returned).
+
         :returns: First :class:`~rattail.db.model.products.Product`
            instance found if there was a match; otherwise ``None``.
         """
-        from rattail.db.api.products import get_product_by_code
+        if not entry:
+            return
 
-        return get_product_by_code(session, entry)
+        model = self.model
+        products = session.query(model.Product).join(model.ProductCode)
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
+        if not include_deleted:
+            products = products.filter(model.Product.deleted == False)
+        return products.filter(model.ProductCode.code == entry).first()
+
+    def search_products(self, session, entry, **kwargs):
+        """
+        Perform a product search across multiple fields, and return
+        results as JSON data rows.
+        """
+        model = self.model
+        final_results = []
+
+        # first we'll attempt "lookup" logic..
+
+        lookup_fields = kwargs.get('lookup_fields', [
+            '_product_key_',
+        ])
+
+        if lookup_fields:
+            product = self.locate_product_for_entry(
+                session, entry, lookup_fields=lookup_fields)
+            if product:
+                final_results.append(product)
+
+        # then we'll attempt "search" logic..
+
+        search_fields = kwargs.get('search_fields', [
+            'item_id',
+            'brand',
+            'description',
+            'size',
+        ])
+
+        searches = {
+            'item_id': self.search_products_for_item_id,
+            'brand': self.search_products_for_brand,
+            'description': self.search_products_for_description,
+            'size': self.search_products_for_size,
+        }
+
+        for field in search_fields:
+            if field in searches:
+                search = searches[field]
+                if search:
+                    products = search(session, entry, **kwargs)
+                    final_results.extend(products)
+            else:
+                log.warning("unknown search field: %s", field)
+
+        return [self.normalize_product(c)
+                for c in final_results]
+
+    def search_products_for_item_id(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            **kwargs):
+        """
+        Search for products where the
+        :attr:`~rattail.db.model.products.Product.item_id` contains
+        the given value.
+
+        :param entry: Search term.
+
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :returns: List of products matching the search.
+        """
+        model = self.model
+        entry = entry.lower()
+
+        products = session.query(model.Product)\
+                          .filter(model.Product.item_id.ilike(f'%{entry}%'))
+
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
+
+        return products.all()
+
+    def search_products_for_brand(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            **kwargs):
+        """
+        Search for products where the brand
+        :attr:`~rattail.db.model.products.Brand.name` contains the
+        given value.
+
+        :param entry: Search term.
+
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :returns: List of products matching the search.
+        """
+        model = self.model
+        entry = entry.lower()
+
+        products = session.query(model.Product)\
+                          .join(model.Brand)\
+                          .filter(model.Brand.name.ilike(f'%{entry}%'))
+
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
+
+        return products.all()
+
+    def search_products_for_description(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            **kwargs):
+        """
+        Search for products where the
+        :attr:`~rattail.db.model.products.Product.description`
+        contains the given value.
+
+        :param entry: Search term.
+
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :returns: List of products matching the search.
+        """
+        model = self.model
+        entry = entry.lower()
+
+        products = session.query(model.Product)\
+                          .filter(model.Product.description.ilike(f'%{entry}%'))
+
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
+
+        return products.all()
+
+    def search_products_for_size(
+            self,
+            session,
+            entry,
+            include_not_for_sale=False,
+            **kwargs):
+        """
+        Search for products where the
+        :attr:`~rattail.db.model.products.Product.size` contains the
+        given value.
+
+        :param entry: Search term.
+
+        :param include_not_for_sale: Optional flag to include items
+           which are "not for sale" in the search results.
+
+        :returns: List of products matching the search.
+        """
+        model = self.model
+        entry = entry.lower()
+
+        products = session.query(model.Product)\
+                          .filter(model.Product.size.ilike(f'%{entry}%'))
+
+        if not include_not_for_sale:
+            products = products.filter(model.Product.not_for_sale == False)
+
+        return products.all()
 
     def is_active_for_store(self, product, store, **kwargs):
         """
@@ -530,6 +800,7 @@ class ProductsHandler(GenericHandler, MergeMixin):
             'description': product.description,
             'size': product.size,
             '_str': str(product),
+            'organic': product.organic,
         }
 
         if not fields:
@@ -537,6 +808,7 @@ class ProductsHandler(GenericHandler, MergeMixin):
                 'brand_name',
                 'full_description',
                 'department_name',
+                'unit_price_display',
             ]
 
         if 'url' in fields:
@@ -679,18 +951,18 @@ class ProductsHandler(GenericHandler, MergeMixin):
         # of saying "unit" - and does not consider i18n etc.
 
         # Each
-        unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_EACH]
+        ea_unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_EACH]
         if not product or not product.weighed:
             choices.append({'key': self.enum.UNIT_OF_MEASURE_EACH,
-                            'value': unit_name})
+                            'value': ea_unit_name})
 
-        # # Pound
-        # if not product or product.weighed:
-        #     unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_POUND]
-        #     choices.append({
-        #         'key': self.enum.UNIT_OF_MEASURE_POUND,
-        #         'value': unit_name,
-        #     })
+        # Pound
+        lb_unit_name = self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_POUND]
+        if not product or product.weighed:
+            choices.append({'key': self.enum.UNIT_OF_MEASURE_POUND,
+                            'value': lb_unit_name})
+
+        unit_name = lb_unit_name if product and product.weighed else ea_unit_name
 
         # Case
         case_text = None
@@ -698,14 +970,11 @@ class ProductsHandler(GenericHandler, MergeMixin):
         if product:
             case_size = self.get_case_size(product)
         if case_size is None:
-            case_text = "{} (&times; ?? {})".format(
-                self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE],
-                unit_name)
+            case_text = (f"{self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE]} "
+                         f"(&times; ?? {unit_name})")
         elif case_size > 1:
-            case_text = "{} (&times; {} {})".format(
-                self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE],
-                self.app.render_quantity(case_size),
-                unit_name)
+            case_text = (f"{self.enum.UNIT_OF_MEASURE[self.enum.UNIT_OF_MEASURE_CASE]} "
+                         f"(&times; {self.app.render_quantity(case_size)} {unit_name})")
         if case_text:
             choices.append({'key': self.enum.UNIT_OF_MEASURE_CASE,
                             'value': case_text})
@@ -766,12 +1035,18 @@ class ProductsHandler(GenericHandler, MergeMixin):
         if upc:
             return pod.get_image_url(self.config, upc)
 
+    def get_last_sold(self, product, store=None, **kwargs):
+        """
+        Fetch the "last sold" date for the given product (and store).
+        """
+        raise NotImplementedError
+
     def render_product_key(self, product, **kwargs):
         """
         Render the key value for the given product, as human-readable
         text.
         """
-        product_key = self.config.product_key()
+        product_key = self.app.get_product_key_field()
         if product_key == 'upc':
             return self.app.render_gpc(product.upc)
         return str(getattr(product, product_key))
@@ -829,8 +1104,11 @@ class ProductsHandler(GenericHandler, MergeMixin):
         :param user: Reference to the User responsible.
         """
         custorder_handler = self.app.get_custorder_handler()
-        custorder_handler.resolve_product(pending, product, user)
+        custorder_handler.resolve_product(pending, product, user, **kwargs)
 
+        pending.product = product
+        pending.resolved = self.app.make_utc()
+        pending.resolved_by = user
         pending.status_code = self.enum.PENDING_PRODUCT_STATUS_RESOLVED
 
     def get_uom_sil_codes(self, session, uppercase=False, **kwargs):
@@ -875,6 +1153,9 @@ class ProductsHandler(GenericHandler, MergeMixin):
         :returns: SIL code for the UOM, as string (e.g. ``'49'``), or ``None``
            if no matching code was found.
         """
+        import sqlalchemy as sa
+        from sqlalchemy import orm
+
         model = self.model
         query = session.query(model.UnitOfMeasure)
         if uppercase:
@@ -929,6 +1210,34 @@ class ProductsHandler(GenericHandler, MergeMixin):
         """
         return []
 
+    def make_future_cost_current(self, future, **kwargs):
+        """
+        Take the given future cost, and make it current.
+        """
+        if not future.cost:
+            raise NotImplementedError("don't know how to create costs yet")
+
+        cost = future.cost
+
+        if not future.product:
+            raise ValueError("future cost does not have product association: {}".format(future))
+
+        if not future.vendor:
+            raise ValueError("future cost does not have vendor association: {}".format(future))
+
+        if future.product is not cost.product:
+            raise ValueError("product mismatch for future/current costs: {}".format(future))
+
+        if future.vendor is not cost.vendor:
+            raise ValueError("vendor mismatch for future/current costs: {}".format(future))
+
+        for field in future_cost_fieldmap:
+            setattr(cost, future_cost_fieldmap[field], getattr(future, field))
+
+        session = self.app.get_session(future)
+        session.delete(future)
+        log.debug("future cost became current: %s", cost)
+
     def get_merge_preview_fields(self, **kwargs):
         F = self.make_merge_field
         return [
@@ -973,9 +1282,10 @@ def get_products_handler(config, **kwargs):
     """
     Create and return the configured :class:`ProductsHandler` instance.
     """
+    app = config.get_app()
     spec = config.get('rattail', 'products.handler')
     if spec:
-        factory = load_object(spec)
+        factory = app.load_object(spec)
     else:
         factory = ProductsHandler
     return factory(config, **kwargs)
