@@ -34,7 +34,8 @@ import sys
 import colander
 from mako.template import Template
 
-from rattail.util import get_class_hierarchy, get_studly_prefix
+from wuttjamaican.util import get_class_hierarchy
+from rattail.util import get_studly_prefix
 from rattail.mako import ResourceTemplateLookup
 
 
