@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,18 +24,10 @@
 Report Definitions
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import re
 
-import six
 
-from rattail.db import cache
-from rattail.util import progress_loop
-
-
-@six.python_2_unicode_compatible
-class Report(object):
+class Report:
     """
     Base class for all reports.
 
@@ -68,8 +60,8 @@ class Report(object):
     def __init__(self, config, **kwargs):
         self.config = config
         self.app = config.get_app()
+        self.model = self.app.model
         self.enum = config.get_enum()
-        self.model = config.get_model()
         for key, value in kwargs.items():
             setattr(self, key, value)
 
@@ -157,10 +149,10 @@ class Report(object):
         raise NotImplementedError
 
     def cache_model(self, session, model, **kwargs):
-        return cache.cache_model(session, model, **kwargs)
+        return self.app.cache_model(session, model, **kwargs)
 
     def progress_loop(self, func, items, factory, **kwargs):
-        return progress_loop(func, items, factory, **kwargs)
+        return self.app.progress_loop(func, items, factory, **kwargs)
 
 
 class ReportParam(object):
