@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2018 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,13 +24,10 @@
 SIL Writer
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import datetime
+import warnings
 from decimal import Decimal
 
-import six
-
 from rattail import __version__
 from rattail.core import Object
 from rattail.gpc import GPC
@@ -68,6 +65,9 @@ class Writer(Object):
         """
         Consume and return a SIL-compatible batch ID.
         """
+        warnings.warn("writer.consume_batch_id() method is deprecated; "
+                      "please obtain a batch id another way (e.g. w/ handler)",
+                      DeprecationWarning, stacklevel=2)
         return batches.consume_batch_id(source)
 
     def val(self, value):
@@ -92,20 +92,30 @@ class Writer(Object):
         if value is None:
             return ''
         if isinstance(value, GPC):
-            return six.text_type(value)
+            return str(value)
         if isinstance(value, int):
-            return six.text_type(value)
+            return str(value)
         if isinstance(value, float):
-            return six.text_type(value)
+            return str(value)
         if isinstance(value, Decimal):
-            return six.text_type(value)
+            return str(value)
         if isinstance(value, datetime.date):
             return value.strftime('%Y%j')
         if isinstance(value, datetime.time):
             return value.strftime('%H%M')
-        if not isinstance(value, six.string_types):
-            value = six.text_type(value)
-        return "'{}'".format(value.replace("'", "''"))
+        if not isinstance(value, str):
+            value = str(value)
+
+        # nb. we will be converting \n to \r\n later, when writing
+        # file.  so must cleanup anything that would complicate that
+        value = value.replace('\r\n', '\n')
+
+        # nb. also, if a string value does contain newlines, we must
+        # embed backslash to properly escape them
+        value = value.replace('\n', '\\\n')
+
+        value = value.replace("'", "''")
+        return f"'{value}'"
 
     def write(self, string):
         self.fileobj.write(string)
