@@ -24,15 +24,15 @@
 Trainwreck Config
 """
 
+from wuttjamaican.conf import WuttaConfigExtension
+
 try:
     from rattail.trainwreck.db import Session as TrainwreckSession
 except ImportError:
     TrainwreckSession = None
 
-from rattail.config import ConfigExtension
 
-
-class TrainwreckConfig(ConfigExtension):
+class TrainwreckConfig(WuttaConfigExtension):
     """
     Configures any Trainwreck database connections
     """
