@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,13 +24,10 @@
 Trainwreck Handler
 """
 
-from __future__ import unicode_literals, absolute_import
-
 import warnings
+from collections import OrderedDict
 
 from rattail.app import GenericHandler
-from rattail.util import OrderedDict
-from rattail.trainwreck.db import Session as TrainwreckSession
 
 
 class TrainwreckHandler(GenericHandler):
@@ -70,10 +67,12 @@ class TrainwreckHandler(GenericHandler):
         """
         Make a session for a Trainwreck DB.
         """
+        from rattail.trainwreck.db import Session
+
         if 'bind' not in kwargs:
             engine = self.config.trainwreck_engines[dbkey]
             kwargs['bind'] = engine
-        return TrainwreckSession(**kwargs)
+        return Session(**kwargs)
 
     def get_model(self, **kwargs):
         """
@@ -115,7 +114,7 @@ class TrainwreckHandler(GenericHandler):
                 warnings.warn("[tailbone] 'engines.trainwreck.hidden' is a "
                               "deprecated setting, please use "
                               "[trainwreck.db] 'hide' instead",
-                              DeprecationWarning)
+                              DeprecationWarning, stacklevel=2)
 
         return hidden or []
 
@@ -160,3 +159,13 @@ class TrainwreckHandler(GenericHandler):
                      .first()
         if txn:
             return self.app.localtime(txn.end_time, from_utc=True).date()
+
+    def prune_date(self, session, date, start_time, end_time):
+        """
+        Your custom handler must implement the logic for this method.
+        It is responsible for fully pruning data from the given
+        session's database, for the given date.  The start time and
+        end times given may be used for your queries; they are in
+        localtime and are zone-aware.
+        """
+        raise NotImplementedError
