@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2023 Lance Edgar
+#  Copyright © 2010-2024 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -36,7 +36,9 @@ import warnings
 
 from wuttjamaican.util import (UNSPECIFIED,
                                load_entry_points as wutta_load_entry_points,
-                               load_object as wutta_load_object)
+                               load_object as wutta_load_object,
+                               make_title as wutta_make_title,
+                               get_class_hierarchy as wutta_get_class_hierarchy)
 
 
 # TODO: deprecate / remove this
@@ -48,7 +50,7 @@ class OrderedDict(collections.OrderedDict):
         warnings.warn("rattail.util.OrderedDict is deprecated; "
                       "please use collections.OrderedDict instead",
                       DeprecationWarning, stacklevel=2)
-        super(OrderedDict, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 def capture_output(*args, **kwargs):
@@ -133,37 +135,14 @@ def data_diffs(local_data, host_data, fields=None,
 
 def get_class_hierarchy(klass, topfirst=True):
     """
-    Returns a list of all classes in the inheritance chain for the
-    given class.
-
-    For instance::
-
-       class A(object):
-          pass
-
-       class B(A):
-          pass
-
-       class C(B):
-          pass
-
-       get_class_hierarchy(C)
-       # -> [A, B, C]
-
-    Specify ``topfirst=False`` to get ``[C, B, A]`` instead.
+    DEPRECATED - use
+    :func:`wuttjamaican:wuttjamaican.util.get_class_hierarchy()`
+    instead.
     """
-    hierarchy = []
-
-    def traverse(cls):
-        if cls is not object:
-            hierarchy.append(cls)
-            for parent in cls.__bases__:
-                traverse(parent)
-
-    traverse(klass)
-    if topfirst:
-        hierarchy.reverse()
-    return hierarchy
+    warnings.warn("rattail.util.get_class_hierarchy() is deprecated; "
+                  "please use wuttjamaican.util.get_class_hierarchy() instead",
+                  DeprecationWarning, stacklevel=2)
+    return wutta_get_class_hierarchy(klass, topfirst=topfirst)
 
 
 def get_package_name(name):
@@ -274,14 +253,14 @@ def get_pkg_version(name):
     return version(name)
 
 
+# TODO: deprecate / remove this
 def prettify(text):
     """
-    Return a "prettified" version of text.
+    DEPRECATED - use
+    :meth:`wuttjamaican:wuttjamaican.app.AppHandler.make_title()`
+    instead.
     """
-    text = text.replace('_', ' ')
-    text = text.replace('-', ' ')
-    words = text.split()
-    return ' '.join([x.capitalize() for x in words])
+    return wutta_make_title(text)
 
 
 def pretty_boolean(value):
@@ -306,7 +285,10 @@ def hours_as_decimal(hours, places=2):
 
 
 def pretty_hours(hours=None, seconds=None):
-    """ DEPRECATED """
+    """
+    DEPRECATED - use :meth:`rattail.app.AppHandler.render_duration()`
+    instead.
+    """
     warnings.warn("pretty_hours() function is deprecated; please "
                   "use AppHandler.render_duration() insted",
                   DeprecationWarning, stacklevel=2)
@@ -349,6 +331,47 @@ def pretty_quantity(value, empty_zero=False):
     return str(value).rstrip('0')
 
 
+def render_duration(delta=None, hours=None, seconds=None):
+    """
+    Render a time duration for human eyes, e.g. "1:30" for 1.5 hours.
+
+    Note that you must specify one of: ``delta``, ``hours``, ``seconds``.
+
+    :param delta: If specified, should be a ``datetime.timedelta``
+       object representing the duration.
+
+    :param hours: If specified, should be the number of hours
+       elapsed for the duration, as decimal.
+
+    :param seconds: If specified, should be the number of seconds
+       elapsed for the duration, as integer.
+
+    :returns: Duration rendered as human-friendly string.  Note
+       that if all params are empty, this will return an empty
+       string.
+    """
+    if delta is None and hours is None and seconds is None:
+        return ""
+
+    if delta is None:
+        if seconds is None:
+            seconds = int(hours * 60 * 60)
+        delta = datetime.timedelta(seconds=seconds)
+
+    # calculate minutes for the 'seconds' portion of delta.  note
+    # that we must ensure it is correctly rounded
+    minutes = delta.seconds / 60
+    minutes = decimal.Decimal(str(minutes)).quantize(decimal.Decimal('1'))
+
+    # now add in minutes for the 'days' portion of delta
+    minutes += delta.days * 24 * 60
+
+    # calculate/render final string
+    hours = int(minutes // 60)
+    minutes = int(minutes % 60)
+    return f"{hours}:{minutes:02d}"
+
+
 def progress_loop(func, items, factory, *args, **kwargs):
     """
     This will iterate over ``items`` and call ``func`` for each.  If a progress
