@@ -2,7 +2,7 @@
 ################################################################################
 #
 #  Rattail -- Retail Software Framework
-#  Copyright © 2010-2017 Lance Edgar
+#  Copyright © 2010-2023 Lance Edgar
 #
 #  This file is part of Rattail.
 #
@@ -24,8 +24,6 @@
 Windows Services
 """
 
-from __future__ import unicode_literals
-
 import sys
 import subprocess
 import logging
@@ -162,7 +160,7 @@ def execute_service_command(module, command, *args):
     command = [command]
     if args:
         command = list(args) + command
-    subprocess.call([sys.executable, module.__file__] + command)
+    subprocess.call([sys.executable, '-m', module.__name__] + command)
 
 
 def delayed_auto_start_service(name):
