@@ -31,12 +31,12 @@ from invoke import task
 
 
 @task
-def release(c, tests=False):
+def release(c, skip_tests=False):
     """
     Release a new version of `rattail`.
     """
-    if tests:
-        c.run('tox')
+    if not skip_tests:
+        c.run('pytest')
 
     if os.path.exists('dist'):
         shutil.rmtree('dist')
