@@ -1,86 +1,58 @@
 # -*- coding: utf-8; -*-
 
 from unittest import TestCase
-from unittest.mock import patch, DEFAULT, Mock, MagicMock, call
+from unittest.mock import patch, DEFAULT, Mock, MagicMock
 
 from rattail.config import RattailConfig
+from rattail.testing import DataTestCase
+
 
 try:
     from sqlalchemy import orm
     from rattail import db
+    from rattail.db import changes as mod
     from rattail.db import changes, model
-    from .. import DataTestCase
 except ImportError:
     pass
 else:
 
-    class TestRecordChangesFunc(TestCase):
 
-        def setUp(self):
-            self.config = RattailConfig()
-            self.app = self.config.get_app()
+    class MockRecorder:
+        def __init__(self, config):
+            pass
 
-        def test_session_class(self):
-            Session = orm.sessionmaker()
-            if hasattr(Session, 'kw'):
-                self.assertRaises(KeyError, Session.kw.__getitem__, 'rattail_record_changes')
-            self.assertRaises(AttributeError, getattr, Session, 'rattail_record_changes')
-            changes.record_changes(Session)
-            self.assertTrue(Session.rattail_record_changes)
 
-        def test_session_instance(self):
-            session = db.Session()
-            self.assertFalse(session.rattail_record_changes)
-            changes.record_changes(session)
-            self.assertTrue(session.rattail_record_changes)
-            session.close()
+    class TestRecordChanges(DataTestCase):
 
-        def test_recorder(self):
+        def test_recorder_instance(self):
+            recorder = mod.ChangeRecorder(self.config)
+            self.assertFalse(self.session.rattail_record_changes)
+            mod.record_changes(self.session, recorder=recorder)
+            self.assertTrue(self.session.rattail_record_changes)
 
-            # no recorder
-            session = db.Session()
-            self.assertRaises(AttributeError, getattr, session, 'rattail_change_recorder')
-            session.close()
+        def test_recorder_factory(self):
+            self.assertFalse(self.session.rattail_record_changes)
+            mod.record_changes(self.session, recorder=mod.ChangeRecorder, config=self.config)
+            self.assertTrue(self.session.rattail_record_changes)
 
-            # default recorder
-            session = db.Session()
-            changes.record_changes(session)
-            self.assertIs(type(session.rattail_change_recorder), changes.ChangeRecorder)
-            session.close()
+        def test_configured_recorder(self):
+            self.config.setdefault('rattail.db.changes.recorder', 'tests.db.test_changes:MockRecorder')
+            self.assertFalse(self.session.rattail_record_changes)
+            mod.record_changes(self.session, config=self.config)
+            self.assertTrue(self.session.rattail_record_changes)
 
-            # specify recorder instance
-            recorder = changes.ChangeRecorder(self.config)
-            session = db.Session()
-            changes.record_changes(session, recorder=recorder)
-            self.assertIs(session.rattail_change_recorder, recorder)
-            session.close()
+        def test_default_recorder(self):
+            self.assertFalse(self.session.rattail_record_changes)
+            mod.record_changes(self.session, config=self.config)
+            self.assertTrue(self.session.rattail_record_changes)
 
-            # specify recorder factory
-            session = db.Session()
-            changes.record_changes(session, recorder=changes.ChangeRecorder, config=self.config)
-            self.assertIs(type(session.rattail_change_recorder), changes.ChangeRecorder)
-            session.close()
-
-            # specify recorder spec via config
-            config = RattailConfig()
-            config.setdefault('rattail.db', 'changes.recorder', 'rattail.db.changes:ChangeRecorder')
-            session = db.Session()
-            changes.record_changes(session, config=config)
-            self.assertIs(type(session.rattail_change_recorder), changes.ChangeRecorder)
-            session.close()
-
-            # invalid recorder
-            session = db.Session()
-            self.assertRaises(ValueError, changes.record_changes, session, recorder='bogus')
-            session.close()
+        def test_invalid_recorder(self):
+            self.assertFalse(self.session.rattail_record_changes)
+            self.assertRaises(ValueError, mod.record_changes, self.session, recorder=42)
 
 
     class TestChangeRecorder(DataTestCase):
 
-        def extra_setup(self):
-            self.config = RattailConfig()
-            self.app = self.config.get_app()
-
         def test_ignore_object(self):
             recorder = changes.ChangeRecorder(self.config)
             self.assertTrue(recorder.ignore_object(model.Setting()))
@@ -311,8 +283,7 @@ else:
     class TestFunctionalChanges(DataTestCase):
 
         def setUp(self):
-            super().setUp()
-            self.config = RattailConfig()
+            self.setup_db()
             changes.record_changes(self.session, config=self.config)
 
         def test_add(self):
