@@ -11,20 +11,29 @@ import pytest
 from wuttjamaican.exc import ConfigurationError
 
 from rattail import app as mod
-from rattail.config import make_config
+from rattail.config import RattailConfig
 from rattail.core import Object
 from rattail.db import Session
 from rattail.autocomplete import Autocompleter
 from rattail.batch import BatchHandler
-from rattail.bouncer import BounceHandler
 from rattail.importing import ImportHandler
 from rattail.gpc import GPC
 
 
+try:
+    from rattail.bouncer import BounceHandler
+except ImportError:
+    pass
+else:
+
+    class FooBarBounceHandler(BounceHandler):
+        pass
+
+
 class TestAppHandler(TestCase):
 
     def setUp(self):
-        self.config = make_config([], extend=False)
+        self.config = RattailConfig()
         self.app = mod.AppHandler(self.config)
         self.config.app = self.app
 
@@ -73,6 +82,36 @@ class TestAppHandler(TestCase):
         self.assertEqual(self.app.get_title(), "Bar")
         self.assertEqual(self.app.get_title(default="Foo"), "Bar")
 
+    def test_get_version(self):
+        from importlib.metadata import version
+
+        try:
+            from sqlalchemy.orm import Query
+        except ImportError:
+            pytest.skip("test is not relevant without sqlalchemy")
+
+        # works with "non-native" objects
+        query = Query({})
+        ver = self.app.get_version(obj=query)
+        self.assertEqual(ver, version('SQLAlchemy'))
+
+        # can override dist via config
+        self.config.setdefault('rattail.app_dist', 'SQLAlchemy')
+        ver = self.app.get_version()
+        self.assertEqual(ver, version('SQLAlchemy'))
+
+        # but the provided object takes precedence
+        ver = self.app.get_version(obj=query)
+        self.assertEqual(ver, version('SQLAlchemy'))
+
+        # reset
+        del self.config.defaults['rattail.app_dist']
+
+        # can also override package via config
+        self.config.setdefault('rattail.app_package', 'mako')
+        ver = self.app.get_version()
+        self.assertEqual(ver, version('Mako'))
+
     def test_get_timezone(self):
 
         # unconfigured zone causes error
@@ -221,6 +260,11 @@ class TestAppHandler(TestCase):
 
     def test_get_bounce_handler(self):
 
+        try:
+            from rattail.bouncer import BounceHandler
+        except ImportError:
+            pytest.skip("test not relevant without flufl.bounce")
+
         # unknown type raises error by default
         self.assertRaises(ValueError, self.app.get_bounce_handler, 'foobar')
 
@@ -797,10 +841,6 @@ class FooBarBatchHandler(BatchHandler):
     pass
 
 
-class FooBarBounceHandler(BounceHandler):
-    pass
-
-
 class FromFooToBar(ImportHandler):
     host_key = 'rattail'
     local_key = 'rattail'
