@@ -2,7 +2,7 @@
 
 from unittest import TestCase
 
-from rattail.config import make_config
+from rattail.config import RattailConfig
 
 try:
     import sqlalchemy as sa
@@ -15,13 +15,10 @@ else:
     class TestAuthHandler(TestCase):
 
         def setUp(self):
-            self.config = self.make_config()
+            self.config = RattailConfig()
             self.app = self.config.get_app()
             self.handler = self.make_handler()
 
-        def make_config(self):
-            return make_config([], extend=False)
-
         def make_handler(self):
             return mod.AuthHandler(self.config)
 
