@@ -3,7 +3,7 @@
 from unittest import TestCase
 
 from rattail import clientele
-from rattail.config import make_config
+from rattail.config import RattailConfig
 
 try:
     import sqlalchemy as sa
@@ -14,7 +14,7 @@ else:
     class TestClienteleHandler(TestCase):
 
         def setUp(self):
-            self.config = make_config([], extend=False)
+            self.config = RattailConfig()
             self.app = self.config.get_app()
             self.handler = clientele.ClienteleHandler(self.config)
             self.engine = sa.create_engine('sqlite://')
