@@ -4,6 +4,7 @@ import configparser
 import datetime
 import os
 import sys
+import tempfile
 from unittest import TestCase
 from unittest.mock import patch
 
@@ -12,12 +13,37 @@ import pytest
 from wuttjamaican.testing import FileConfigTestCase
 from wuttjamaican.exc import ConfigurationError
 
-from rattail import config
+from rattail import config as mod, db
 from rattail.app import AppHandler
 
 
 class TestRattailConfig(FileConfigTestCase):
 
+    def make_config(self, **kwargs):
+        return mod.RattailConfig(**kwargs)
+
+    def test_constructor(self):
+
+        # no db by default
+        config = self.make_config()
+        if db.Session:
+            session = db.Session()
+            self.assertIsNone(session.bind)
+            # nb. session also has our config now
+            self.assertIs(session.rattail_config, config)
+        else:
+            # no sqlalchemy, so no rattail engines
+            self.assertFalse(hasattr(config, 'appdb_engines'))
+            self.assertFalse(hasattr(config, 'appdb_engine'))
+
+        # default db
+        config = self.make_config(defaults={
+            'rattail.db.default.url': 'sqlite://',
+        })
+        if db.Session:
+            session = db.Session()
+            self.assertEqual(str(session.bind.url), 'sqlite://')
+
     def test_prioritized_files(self):
         first = self.write_file('first.conf', """\
 [foo]
@@ -29,7 +55,7 @@ bar = 1
 require = %(here)s/first.conf
 """)
 
-        myconfig = config.RattailConfig(files=[second])
+        myconfig = self.make_config(files=[second])
         files = myconfig.prioritized_files
         self.assertEqual(len(files), 2)
         self.assertEqual(files[0], second)
@@ -42,13 +68,13 @@ require = %(here)s/first.conf
             pytest.skip("test is not relevant without sqlalchemy")
 
         # default func
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertEqual(myconfig.default_engine_maker_spec, 'rattail.db.config:make_engine_from_config')
         make_engine = myconfig.get_engine_maker()
         self.assertIs(make_engine, make_engine_from_config)
 
     def test_setdefault(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # nb. the tests below are effectively testing the custom get()
         # method in addition to setdefault()
@@ -75,7 +101,7 @@ require = %(here)s/first.conf
         self.assertRaises(ValueError, myconfig.get, 'foo', 'bar', 'blarg', 'blast')
 
     def test_get(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         myconfig.setdefault('foo.bar', 'baz')
 
         # can pass section + option
@@ -89,13 +115,13 @@ require = %(here)s/first.conf
         self.assertRaises(ValueError, myconfig.get)
 
     def test_getbool(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertFalse(myconfig.getbool('foo.bar'))
         myconfig.setdefault('foo.bar', 'true')
         self.assertTrue(myconfig.getbool('foo.bar'))
 
     def test_get_date(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertIsNone(myconfig.get_date('foo.date'))
         myconfig.setdefault('foo.date', '2023-11-20')
         value = myconfig.get_date('foo.date')
@@ -103,7 +129,7 @@ require = %(here)s/first.conf
         self.assertEqual(value, datetime.date(2023, 11, 20))
 
     def test_getdate(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertIsNone(myconfig.getdate('foo.date'))
         myconfig.setdefault('foo.date', '2023-11-20')
         value = myconfig.getdate('foo.date')
@@ -111,29 +137,29 @@ require = %(here)s/first.conf
         self.assertEqual(value, datetime.date(2023, 11, 20))
 
     def test_getint(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertIsNone(myconfig.getint('foo.bar'))
         myconfig.setdefault('foo.bar', '42')
         self.assertEqual(myconfig.getint('foo.bar'), 42)
 
     def test_getlist(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertIsNone(myconfig.getlist('foo.bar'))
         myconfig.setdefault('foo.bar', 'hello world')
         self.assertEqual(myconfig.getlist('foo.bar'), ['hello', 'world'])
 
     def test_parse_bool(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertTrue(myconfig.parse_bool('true'))
         self.assertFalse(myconfig.parse_bool('false'))
 
     def test_parse_list(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         self.assertEqual(myconfig.parse_list(None), [])
         self.assertEqual(myconfig.parse_list('hello world'), ['hello', 'world'])
 
     def test_make_list_string(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         value = myconfig.make_list_string(['foo', 'bar'])
         self.assertEqual(value, 'foo, bar')
@@ -145,18 +171,18 @@ require = %(here)s/first.conf
         self.assertEqual(value, "\"you don't\", say")
 
     def test_get_app(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         app = myconfig.get_app()
         self.assertIsInstance(app, AppHandler)
         self.assertIs(type(app), AppHandler)
 
     def test_beaker_invalidate_setting(self):
         # TODO: this doesn't really test anything, just gives coverage
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         myconfig.beaker_invalidate_setting('foo')
 
     def test_node_type(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if node type not defined
         self.assertRaises(ConfigurationError, myconfig.node_type)
@@ -175,18 +201,18 @@ require = %(here)s/first.conf
             pytest.skip("test is not relevant without sqlalchemy")
 
         # default is rattail.db.model
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         model = myconfig.get_model()
         self.assertIs(model, sys.modules['rattail.db.model'])
 
         # or config may specify
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
         myconfig.setdefault('rattail.model', 'rattail.trainwreck.db.model')
         model = myconfig.get_model()
         self.assertIs(model, sys.modules['rattail.trainwreck.db.model'])
 
     def test_get_enum(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # default is rattail.enum
         enum = myconfig.get_enum()
@@ -204,7 +230,7 @@ require = %(here)s/first.conf
         except ImportError:
             pytest.skip("test is not relevant without sqlalchemy")
 
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if not defined
         self.assertRaises(ConfigurationError, myconfig.get_trainwreck_model)
@@ -215,7 +241,7 @@ require = %(here)s/first.conf
         self.assertIs(model, sys.modules['rattail.trainwreck.db.model'])
 
     def test_versioning_enabled(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # false by default
         self.assertFalse(myconfig.versioning_enabled())
@@ -225,7 +251,7 @@ require = %(here)s/first.conf
         self.assertTrue(myconfig.versioning_enabled())
 
     def test_app_package(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if not defined
         self.assertRaises(ConfigurationError, myconfig.app_package)
@@ -238,7 +264,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.app_package(), 'bar')
 
     def test_app_title(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # default title
         self.assertEqual(myconfig.app_title(), 'Rattail')
@@ -248,7 +274,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.app_title(), 'Foo')
 
     def test_node_title(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # default title
         self.assertEqual(myconfig.app_title(), 'Rattail')
@@ -258,7 +284,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.node_title(), 'Foo (node)')
 
     def test_running_from_source(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # false by default
         self.assertFalse(myconfig.running_from_source())
@@ -268,7 +294,7 @@ require = %(here)s/first.conf
         self.assertTrue(myconfig.running_from_source())
 
     def test_demo(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # false by default
         self.assertFalse(myconfig.demo())
@@ -278,7 +304,7 @@ require = %(here)s/first.conf
         self.assertTrue(myconfig.demo())
 
     def test_appdir(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # can be none if required is false
         self.assertIsNone(myconfig.appdir(require=False))
@@ -294,7 +320,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.appdir(), '/foo/bar/baz')
 
     def test_datadir(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if not defined
         self.assertRaises(ConfigurationError, myconfig.datadir)
@@ -307,7 +333,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.datadir(), '/foo/bar/baz')
 
     def test_workdir(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if not defined
         self.assertRaises(ConfigurationError, myconfig.workdir)
@@ -320,7 +346,7 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.workdir(), '/foo/bar/baz')
 
     def test_batch_filedir(self):
-        myconfig = config.RattailConfig()
+        myconfig = self.make_config()
 
         # error if not defined
         self.assertRaises(ConfigurationError, myconfig.batch_filedir)
@@ -334,6 +360,44 @@ require = %(here)s/first.conf
         self.assertEqual(myconfig.batch_filedir(key='bar'), os.path.join(path, 'bar'))
 
 
+class TestLegacyConfigExtensionBase(TestCase):
+
+    def test_basic(self):
+        # sanity / coverage check
+        ext = mod.ConfigExtension()
+
+
+class TestRattailConfigExtension(TestCase):
+
+    def make_config(self, **kwargs):
+        return mod.RattailConfig(**kwargs)
+
+    def make_extension(self):
+        return mod.RattailConfigExtension()
+
+    def test_configure(self):
+
+        # no import config yet
+        config = self.make_config()
+        self.assertEqual(config.defaults, {})
+        self.assertIsNone(config.get('rattail.importing.to_rattail.from_csv.import.default_handler'))
+
+        # extension adds import config
+        ext = self.make_extension()
+        ext.configure(config)
+        spec = config.get('rattail.importing.to_rattail.from_csv.import.default_handler')
+        self.assertIsNotNone(spec)
+
+        # poser dir added to path
+        self.assertNotIn('/tmp/foo', sys.path)
+        tempdir = tempfile.mkdtemp()
+        config.setdefault('rattail.poser', tempdir)
+        ext.configure(config)
+        self.assertIn(tempdir, sys.path)
+        sys.path.remove(tempdir)
+        os.rmdir(tempdir)
+
+
 class TestRattailDefaultFiles(FileConfigTestCase):
 
     def test_quiet_conf(self):
@@ -348,14 +412,14 @@ class TestRattailDefaultFiles(FileConfigTestCase):
 
                 # generic files by default
                 mockos.path.exists.return_value = False
-                files = config.rattail_default_files('rattail')
+                files = mod.rattail_default_files('rattail')
                 generic_default_files.assert_called_once_with('rattail')
                 self.assertEqual(files, [generic])
 
                 # but if quiet.conf exists, will return that
                 generic_default_files.reset_mock()
                 mockos.path.exists.return_value = True
-                files = config.rattail_default_files('rattail')
+                files = mod.rattail_default_files('rattail')
                 generic_default_files.assert_not_called()
                 self.assertEqual(files, [quiet])
 
@@ -367,18 +431,18 @@ class TestMakeConfig(FileConfigTestCase):
         myfile = self.write_file('my.conf', '')
 
         # generic files by default
-        myconfig = config.make_config(default_files=[generic])
+        myconfig = mod.make_config(default_files=[generic])
         self.assertEqual(myconfig.files_read, [generic])
 
         # can specify single primary file
-        myconfig = config.make_config(myfile, default_files=[generic])
+        myconfig = mod.make_config(myfile, default_files=[generic])
         self.assertEqual(myconfig.files_read, [myfile])
 
         # can specify primary files as list
-        myconfig = config.make_config([myfile], default_files=[generic])
+        myconfig = mod.make_config([myfile], default_files=[generic])
         self.assertEqual(myconfig.files_read, [myfile])
 
         # can specify primary files via env
-        myconfig = config.make_config(env={'RATTAIL_CONFIG_FILES': myfile},
-                                      default_files=[generic])
+        myconfig = mod.make_config(env={'RATTAIL_CONFIG_FILES': myfile},
+                                   default_files=[generic])
         self.assertEqual(myconfig.files_read, [myfile])
