@@ -8,16 +8,13 @@ from unittest import TestCase
 import openpyxl
 
 from rattail import excel as mod
-from rattail.config import make_config
+from rattail.config import RattailConfig
 
 
 class TestExcelReaderXLSX(TestCase):
 
     def setUp(self):
-        self.config = self.make_config()
-
-    def make_config(self):
-        return make_config([], extend=False)
+        self.config = RattailConfig()
 
     def test_strip_fieldnames(self):
         app = self.config.get_app()
