@@ -5,7 +5,7 @@ from unittest import TestCase
 import pytest
 
 from rattail import products as mod
-from rattail.config import make_config
+from rattail.config import RattailConfig
 from rattail.gpc import GPC
 from rattail.db import Session
 
@@ -13,13 +13,10 @@ from rattail.db import Session
 class TestProductsHandler(TestCase):
 
     def setUp(self):
-        self.config = self.make_config()
+        self.config = RattailConfig()
         self.app = self.config.get_app()
         self.handler = self.make_handler()
 
-    def make_config(self):
-        return make_config([], extend=False)
-
     def make_handler(self):
         return mod.ProductsHandler(self.config)
 
