@@ -7,13 +7,13 @@ import pytz
 from wuttjamaican.exc import ConfigurationError
 
 from rattail import time
-from rattail.config import make_config
+from rattail.config import RattailConfig
 
 
 class TestLocaltime(TestCase):
 
     def setUp(self):
-        self.config = make_config([], extend=False)
+        self.config = RattailConfig()
         self.config.setdefault('rattail', 'timezone.default', 'America/Los_Angeles')
         self.config.setdefault('rattail', 'timezone.missouri', 'America/Chicago')
 
@@ -52,7 +52,7 @@ class TestLocaltime(TestCase):
 class TestTimezone(TestCase):
 
     def setUp(self):
-        self.config = make_config([], extend=False)
+        self.config = RattailConfig()
 
     def test_default_timezone_returned_by_default(self):
         self.config.setdefault('rattail', 'timezone.default', 'Antarctica/Rothera')
